/*++++++++++++++
.IDENTIFICATION anafile.c
.LANGUAGE       C
.AUTHOR         Francois Ochsenbein [CDS]
.ENVIRONMENT    Unix, CDS Catalogues
.KEYWORDS       Table, FITS
.VERSION  1.0   15-May-1991: First creation (mods 1.1 to 1.9 deleted)
.VERSION  2.0   15-Mar-1993: -f and derived facilities (2.1 to 2.9 deleted)
.VERSION  3.0   09-Nov-1993: Better output for -cc ;
.VERSION  4.0   02-Aug-1994: New package
.VERSION  4.1   16-Feb-1996: " . " is an error ...
.VERSION  4.2   26-Jun-1996: Always compare with preceding...
.VERSION  4.3   23-Oct-1996: BUG fixed, gcc too strict about strings constants..
.VERSION  4.4   27-Mar-1997: -t option, take care of directories...
.VERSION  4.5   02-Apr-1997: For directories: list size of it...
.VERSION  4.6   07-Aug-1997: Error, blank lines were not correctly checked...
.VERSION  4.7   19-Nov-1997: -f1X option
.VERSION  4.8   18-Sep-1998: Check last char of file....
.VERSION  4.9   09-May-1999: For bulk e.g. gif, Nrec=1
.VERSION  5.0   23-Feb-2000: Check References
.VERSION  5.1   17-Apr-2000: Bug in references checking
.VERSION  5.2   20-Jul-2000: -i option
.VERSION  5.3   29-Oct-2000: Accept dates with 0000 for NULL
.VERSION  5.4   05-Sep-2001: Verification of references: ignore leading blanks
.VERSION  6.0   18-Sep-2001; -d option (analyze TSV/CSV tables)
.VERSION  6.1   09-Nov-2001: Accept "---" "~" as NULL values.
.VERSION  6.1   13-May-2002: Edit the 'acut' proposals when -d option.
.VERSION  6.2   04-Dec-2002: Accept empty lines -- just write a warning
.VERSION  6.21  01-Jan-2003: Accept () in references
.VERSION  6.22  12-Jan-2003: Compute min/max in -ccg option
.VERSION  6.23  30-Jan-2003: Line position includes the empty line
.VERSION  6.24  27-Feb-2003: BUG ! When just one field exist, was taken as empty
.VERSION  6.3   30-Dec-2003: Accept NULL reference
.VERSION  6.4   21-Jul-2004: Accept "Ref[s()]*" indicating references.
.VERSION  6.5   10-Aug-2004: Edition of error messages with column number
.VERSION  6.51  02-Oct-2004: Count empty lines separately
.VERSION  6.52  15-Apr-2005: Edit always the column in which the error is found.
.VERSION  6.6   29-May-2005: Exponential notation accepted. Headlines options
.VERSION  6.7   18-Jul-2005: Bug in headbuf / Added headlines=symbol
			Accept with numeric F_NULL
			all '-' '*' '~'
.VERSION  6.71   27-Jul-2005: Problem communicated by Sergey Koposov:
			empty lines are not counted in txt files...
.VERSION  6.72   29-Jul-2005: Multiple files are studied individually
.VERSION  6.73   30-Aug-2005: FITS analysis simplified
.VERSION  6.8    10-Sep-2005: Takes directory name into account
.VERSION  6.9    06-Mar-2006: Linux read-only memory problems.
.VERSION  7.0    11-Apr-2006: Option -ccg without delimiter to propose a format
.VERSION  7.1    31-Oct-2006: Accept large subdirctories.
.VERSION  7.2    16-Nov-2006: headlines are in fact COMMENT lines
.VERSION  7.21   01-Dec-2006: acut with comment char
.VERSION  7.22   01-Dec-2006: -q option
.VERSION  7.23   02-Jun-2007: Cosmetics (-Wall)
.VERSION  7.24   21-Aug-2007: different edition of head lines
.VERSION  7.25   27-Feb-2008: -f2 (errors in stderr)
.VERSION  7.26   27-Feb-2009: Pb estimation floating-point number
.VERSION  7.27   08-May-2009: Bug in case of column always empty
.VERSION  7.28   19-Dec-2009: 64-bit
.VERSION  7.33   13-Apr-2010: Accept merged options like -la
.VERSION  7.34   30-Jul-2010: Count the nulls
.VERSION  7.35   20-Jun-2011: Check b_ and B_ (bounds agains main column)
.VERSION  7.36   21-Jun-2011: Pb null vs b_ and B_ (syndrome: bad value)
.VERSION  7.37   25-Jun-2011: Bug when b_ or B_ is null, but not value
.VERSION  7.38   30-Jul-2011: Bug when b_ or B_ is negative
.VERSION  7.39   26-Apr-2012: With -ccg option, propose to trim blanks
.VERSION  7.4    01-May-2012: More details with -v
.VERSION  7.41   12-May-2012: Take care of Inf, NaN, null for numbers
.VERSION  7.42   05-Jun-2012: Check b_ and B_ : take care of NULL4 !
.VERSION  7.43   16-Jul-2012: for TSV, accept names are not the first line
.VERSION  7.44   03-Oct-2012: Pg edition in verbose mode
.VERSION  7.45   19-Nov-2012: No "Stop" message for some users
.VERSION  7.46   13-Dec-2012: Pb when b_col or B_col is null...
.VERSION  7.47   20-Dec-2012: Pb with badly formatted ReadMe
.VERSION  7.48   11-Mar-2013: Pb 64-bit for all numbers
.VERSION  7.49   24-Mar-2013: Pb 64-bit for range computation
.VERSION  7.50   11-Jun-2013: Rewrite: add the headlines
.VERSION  7.51   01-Jul-2013: Trim the line before checking
.VERSION  7.52   01-Aug-2013: Bug fix
.VERSION  7.53   02-Oct-2013: homogenize vizcat with vizin 
.VERSION  7.54   26-Nov-2013: Edition of [min/max] (instead of [min,max]
.VERSION  7.55   22-may-2014: new version (fits2a,..)
.VERSION  7.56   22-jul-2014: new version  'tabmap.c, vizcat.c)
.COMMENTS       Tries to guess which kind of file is stdin.
		Conforms to the document:
		"Astronomical Catalogues at CDS: Adopted Standards", Ver 1.3
---------------*/

#define VERSION "7.54 (2013-11-26)"

#include <catstd.h>
#include <filutil.h>
#include <fcntl.h>		/* File Control Definitions */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <strings.h>
#ifndef O_BINARY		/* Assume Unix System	    */
# define O_BINARY 0
# ifndef DIRENT
# define DIRENT 0
# endif
# include <sys/stat.h>
# if DIRENT
# include <dirent.h>		/* System V 	*/
# else
# include <sys/dir.h>		/* BSD		*/
# define dirent direct
# endif
#endif

typedef int (*SORT_FCT)(/* const void *, const void * */) ;

#define LRECL   512      /* Default LRECL        */
#define MAXW    (MAXBYTES/129/sizeof(size_t))    /* Maxi Column Statistics */

#ifdef DEBUG
static char inbyte;
#endif

static COLINFO end_format;	/* End of Line   */

static char fclass[256];/* Classification of characters	*/
static int8 counts[_NL_+1];     /* Counts of categories */

#ifndef MAX
#define MAX(x,y)        ((x) >= (y) ? (x) : (y))
#define MIN(x,y)        ((x) >= (y) ? (y) : (x))
#endif
#define ITEMS(a)        sizeof(a)/sizeof(a[0])
#ifndef NULL4
#define NULL4 0x80000000
#endif
#ifndef NULL8
#define NULL8 0x8000000000000000LL
#endif

#define Xchar(c)        ((c) & 0x80 ? 128 : (c) & 127)
#define E8		100000000LL	/* 100 millions */

static CATINFO *tinfo;
static TABINFO *tform;
static char *dReadMe;		/* Directory containing the ReadMe */
static char *table_schema;
static char *infile_name = (char *)0;
static  int fop    = 0;		/* Format Options*/

static char verbop = 0;  /* Verbose option       */
static char ofull  = 0;  /* -l option            */
static char oa     = 0;  /* -a option            */
static char oq     = 0;  /* -q option            */
static char ou     = 0;  /* -u option            */
static char oi     = 0;  /* -i option            */
static char ocount = 0;  /* -cc option           */
static char occ    = 0;  /* -cc option           */
static int  error_1;		/* First error   */
static int count_nullrefs = 0;	/* check_refs	 */	/* Unused */
static int count_refcols  = 0;	/* check_refs	 */	/* Unused */

static int errors = 0;  /* Errors counter       */
static char *previous_value;	/* check_order	*/

#define nls     counts[_NL_]    /* NEWLINE characters   */
#define MAX_INT	2147483647.	/* 2**31 - 1	  */
static char   colsep ;		/* -d option	  */
static size_t reclen = 0;  	/* Longest line   */
static int    lsize  = 0;  	/* Current offset */
static int    colno;		/* Current Column */
static int    fmtno;		/* Current Column */
static int    colstart[7];	/* Current Column */
#define colstart1 colstart[1]   /* 1rst non-blank */
#define colstart2 colstart[2]   /* Last non-blank */
#define colstart3 colstart[3]   /* Decimal point  */
#define colstart4 colstart[4]   /* Exponential notation (F_DBLE) */
#define colstart5 colstart[5]   /* Invalid in number    (F_CHAR) */
#define colstart6 colstart[6]   /* Sign position  */
static char   colcontent[40];	/* For min/max	  */
static int    icolcontent;	/* Position ...	  */
static int    headlines = 0;	/* -headline      */
static char   *headexp;		/* -headline 	  */
#define has_head (headexp || headlines)
static int4   lineno;		/* Line number	  */
static char   *headbuf;		/* Text of Headers*/
static int    lheadbuf;		/* Len. of headbuf*/

static int nfields = 0;		/* Allocated Fields for -cc Statistics */
typedef struct {
    double min;			/* Minimal values */
    double max;			/* Maximal values */
    short   dec;		/* Number of decimals  */
    short   pec;		/* Pecularities F=0 1=A 2=I F_NULL F_DBLE */
#define F__nan	0x1000		/* has "NaN" written out */
#define F__nul	0x2000		/* has "null" written out */
#define F__inf	0x4000		/* has "Inf" written out */
#define F__min	0x8000		/* has '---' as null    */
    int    wid ;		/* Column Width   */
    int    wnb ;		/* Column Width without blanks */
    int    null;		/* Count of NULL values */
    char *name;			/* Column name */
    char *unit;			/* Column unit */
} FIELDSTAT;
static FIELDSTAT *fieldstat;
#define colwid(j)	fieldstat[j].wid
#define colwnb(j)	fieldstat[j].wnb
#define colmin(j)	fieldstat[j].min
#define colmax(j)	fieldstat[j].max
#define coldec(j)	fieldstat[j].dec
#define colpec(j)	fieldstat[j].pec
#define colname(j)	fieldstat[j].name
#define colnull(j)	fieldstat[j].null
#define colunit(j)	fieldstat[j].unit

/* static int ncols = 0 ;	      -- Number of columns (-d) */
static size_t *ccounts = (size_t *)0; /* Column Counts */
static size_t *pcounts = (size_t *)0; /* Current Count, used for qsort */
static int lcount   = LRECL;    /* Maxi Columns  */

static struct {
    int nr ; 	/* Number of References */
    short lr ;	/* Length of 1 item	*/
    char rint ;	/* 1 if references are integer	*/
    int (*diff)() ;	/* Comparison function  */
    char *ref ;		/* Space for 1 reference */
    char *refs ;
} references ;

typedef struct { 	/* To save the "more" strings and restore them */
    int items;
    TABINFO *table;
    char *saved_more[2];	/* In fact, count of elements = 'items' */
} SAVEDmore;

/* Error messages / formats */
static char err_bad_fmt[] = "++++Bad Format <%s>";
static char err_bad_dec[] = "++++Bad decimal point <%s> in column '%s'";
static char err_bad_car[] = "++++Char. forbidden   <%s> in column '%s'";
static char err_mis_dig[] = "++++No digit in <%s> column '%s'";
static char err_non_bla[] = "++++Non-blank <%s>";
static char err_non_dig[] = "++++Non-digit char.   <%s> in column '%s'";
static char err_too_lon[] = "++++Too long line";
static char err_bad_eq_[] = "++++Value <%s>  equals preceding in column '%s'";
static char err_bad_val[] = "++++Value <%s> in column '%s'" ;
static char err_not_dec[] = "++++Value <%s> is not decreasing in column '%s'";
static char err_not_inc[] = "++++Value <%s> is not increasing in column '%s'";
static char err_bad_nul[] = "++++Value <%sNULL> in column '%s'";
static char err_too_large[] = "++++Value <%s> too large in column '%s'";
static char err_too_small[] = "++++Value <%s> too small in column '%s'";
static char errf_too_large[] = "++++Value <%g> too large(%g) in column '%s'";
static char errf_too_small[] = "++++Value <%g> too small(%g) in column '%s'";

static char tusage[] = "\
Usage: anafile [-v|-q] [-l] [-a] [-dc] [-u] [-cc[g]] [-w width]\n\
       [-headlines[=n]|-hash] [-f... fmt] [-t desc] [[-i] file...]\n\
";
static char thelp[] = "\
     -v: verbose option\n\
     -q: quiet mode (minimise the comments)\n\
     -a: list the position of each non-ascii character\n\
     -d: accept tab-separated or character-separated input\n\
     -l: examine the complete file\n\
     -u: list columns containing always the same character\n\
     -i: indirect list of files (file or stdin contains filenames)\n\
    -cc: compute column statistics (frequencies)\n\
	 generates a basic format template with the g option\n\
    -w width: specifies the column width for bulk (fixed-length) files\n\
       -hash: is a shortcut for   -headlines=#\n\
  -headlines: first line(s) are column name(s), units, ...\n\
              Give number of headlines, or first byte e.g. headlines=#\n\
 -f fmt_file: format is described in fmt_file according to CDS standards\n\
              byte-byte  Format  Unit  Label  *[range]!?=NULL  Explanations\n\
             -fs : fmt_file must conform to the CDS Standards\n\
             -f1 : only the starting byte is specified;\n\
             -f2 : errors written on stderr (default is on stdout)\n\
             -f. : units are not specified\n\
             -fw : rewrite fmt_file\n\
             -fr : compute actual min/max of columns\n\
     -t desc: specifies the table structure in fmt_file\n\
              (default is the description from file name)\n\
        file: file(s) to examine; . (dot) for no file at all.\n\
              Default is all files from fmt_file, or stdin\n\
";

/* Functions from files in this package */
int ebcdic(), fzclose();
void setpgm();
int sec_prhead();

/* Formats with ll */
static char *llform[] = {
#if int8l == 2
 /* [0] */  " %9lld",
 /* [1] */  "++++char o%03o at line %d[%d] = %lld ...",
 /* [2] */  " %9lld END)\n",
 /* [3] */  " x %d : %lld :",
 /* [4] */  " (%lld non-ascii chars)",
 /* [5] */  "-%lld",
 /* [6] */  " [recno=%lld]\n",
 /* [7] */  " %8lld%c"
#else
 /* [0] */  " %9ld",
 /* [1] */  "++++char o%03o at line %d[%d] = %ld ...",
 /* [2] */  " %9ld END)\n",
 /* [3] */  " x %d : %ld :",
 /* [4] */  " (%ld non-ascii chars)",
 /* [5] */  "-%ld",
 /* [6] */  " [recno=%ld]\n",
 /* [7] */  " %8ld%c"
#endif
};

/*==================================================================
		Internal Routines
 *==================================================================*/
static void usage()
/*++++++++++++++++
.PURPOSE  Print out the "Usage" string
.RETURNS  -
-----------------*/
{
  fprintf(stdout, "%s", tusage) ;
}

static void help()
/*++++++++++++++++
.PURPOSE  Detailed help
.RETURNS  -
-----------------*/
{
  int i, sep;
    usage();
    fprintf(stdout, "%s", thelp) ;

    fprintf(stdout, "Returns file type as\n    ");
    for (i=0, sep=' '; i < _NL_; i++, sep='|')
	fprintf(stdout, "%c%s", sep, ftype(i));
    fprintf(stdout, "\n");
}

static char **get_ls(char *wname)
/*++++++++++++++++
.PURPOSE  Get list of file names from a filename expression
.RETURNS  An array (null-terminated)
.REMARKS  Accept large subdorectories (V7.1)
-----------------*/
{
  static char **array;
  int (*test_fct)() = 0;
  FILE *f;
  int i, n;
  char *acmd, *cmd, **a, **e, *p, buf[256];
    if (array) {	/* Free previous list */
	for (a=array; *a; a++) free(*a);
	free(array);
	array = (char **)0;
    }
    cmd = acmd = malloc(10+strlen(wname));
    strcpy(cmd, "ls "); strcat(cmd, wname);	/* Fastest method */

  try_cmd:
    f = popen(cmd, "r");
    if (!f) {
	fprintf(stderr, "****");
	perror(cmd);
	return((char **)0);
    }

    a = e = (char **)0;
    while (fgets(buf, sizeof(buf), f)) {
	for (p = buf + strlen(buf)-1; isspace(*p); p--) *p = 0;
	if (test_fct && (!(*test_fct)(wname, buf))) continue;
	if (a >= e) {
	    i = (a - array);
	    n = i + 1024;
	    array = array ?
		    realloc(array, n*sizeof(char *)) :
		    malloc(n*sizeof(char *)) ;
	    a = array + i;
	    e = array + n-1;
	}
	*(a++) = strdup(buf);
    }
    pclose(f);

    if (a) *a = (char *)0;
    else {		/* V7.1: the shell failed !!! */
	if (!test_fct) {
	    fprintf(stderr, "++++failed to locate files via: %s\n", cmd);
	    test_fct = strmatch;
	    /* Try another method, from find */
	    cmd = "find . -type f -print | sed 's%^[.]/%%' | sed 's/[.]gz$//'";
	    goto try_cmd;
	}
	fprintf(stderr, "****could not execute: %s\n", cmd);
    }

    free(acmd);
    return(array);
}

static FILE *fizopen(char *name)
/*++++++++++++++++
.PURPOSE  Open a file, may be in local directory or in dReadMe
.RETURNS  Opened file / NULL
.REMARKS  All files in same subdirectory !
-----------------*/
{
  static int call = 0;	/* Calling number */
  char full_name[256];
  int len;
  FILE *f;
    call++;
    if (call == 1) {	/* First Call: try local directory */
        if ((f = fzopen(name))) {
	    if (dReadMe) free(dReadMe);
	    dReadMe = (char *)0;
	    return(f);
        }
    }
    if (dReadMe) {
        len = strlen(name) + strlen(dReadMe) + 2;
        if (len>=sizeof(full_name)) {
	    fprintf(stderr, "****Too long name: %s/%s\n", dReadMe, name);
	    return((FILE *)0);
        }
        strcpy(full_name, dReadMe);
        strcat(full_name, "/");
        strcat(full_name, name);
        f = fzopen(full_name);
    }
    else f = fzopen(name);
    return(f);
}
#define fizclose(f)	fzclose(f)

static char *pc(cc)
/*++++++++++++++++
.PURPOSE  Convert a character into printable text
.RETURNS  Printable text
-----------------*/
  int  cc;	/* IN: Char to print */
{
  static char ret[3];

    if (cc < 0)	return("  ");
    if (cc == ' ') return("sp");
    if (isprint(cc)) ret[0] = ' ', ret[1] = cc;
    else switch(cc) {
	case 127:     return("^?");
	case 128:     return("^$");
	default:      ret[0] = '^', ret[1] = cc|0100;
    }
    return(ret);
}

static int memory (cols)
/*++++++++++++++++
.PURPOSE  Allocate or Expand the ccounts array
.RETURNS  Zero (failure) / 1 (success)
-----------------*/
  size_t  cols;	/* IN: How many columns */
{
  int oc ;
  static int limits = 0;

    if (verbop&2) {
	 printf("\n#... Calling memory for %d cols", (int)cols);
	 printf(llform[6], counts[_NL_]+1);
	 fflush(stdout);
    }

    if (cols == 0) {	/* FREE */
  	free (ccounts); ccounts = (size_t *)0;
	return(1);
    }

    if (cols > limits) {
	if (!limits) {	/* First time */
	    if (cols > MAXW) cols = MAXW;
	    limits = cols;
    	}
	else {
	    limits = cols;
	    if (cols > MAXW)
	    	fprintf(stderr, "++++Record Length (%d) limited to %d\n",
	    	(int)cols, (int)MAXW), cols = MAXW;
	}
    }

    if (ccounts) {                /* Must Expand */
	oc = lcount;
	if (oc >= cols) return(1);
	ccounts = (size_t *)realloc(ccounts, cols*129*sizeof(size_t));
	if (! ccounts)  { perror("Expanding ccounts"); terminate((char *)0); }
    }
    else {
	oc = 0;
	ccounts = (size_t *)malloc(cols*129*sizeof(size_t));
    }
    bzero( ccounts + 129*oc, (cols-oc)*129*sizeof(size_t));
    lcount = cols;
    return(1);
}

static int expand_fields (cols)
/*++++++++++++++++
.PURPOSE  Allocate or Expand the Field statistics
.RETURNS  Zero (failure) / 1 (success)
-----------------*/
  size_t  cols;	/* IN: How many fields */
{
    if (verbop && nfields) {
	printf("\n#... expand_fields for %d cols", (int)cols);
	printf(llform[6], counts[_NL_]+1);
	fflush(stdout);
    }

    if (cols > 0xffff) {	/* V6.73: Does not make sense ! */
	fprintf(stderr, "****Too many columns (%d) to be analyzed!\n",
		(int)cols);
	return(0);
    }
    if (cols <= nfields) {
	printf("++++expand_fields(%d): already %d fields allocated!\n",
	    (int)cols, nfields);
	return(0);
    }
    if (nfields>0)
	fieldstat = realloc(fieldstat, sizeof(FIELDSTAT)*(cols));
    else fieldstat = malloc(sizeof(FIELDSTAT)*(cols));
    bzero(&fieldstat[nfields], sizeof(FIELDSTAT)*(cols-nfields));
    nfields = cols;
    return(1);
}

#define change_ccount(text,old,new) if(old!=new) fprintf(stderr, \
	"#...Change ccount[%s] col#%d from %d to %d (recno=%lld)\n", \
	text, colno, old,new,counts[_NL]+1)
/*#define change_ccount(old,new,text) */
static void icount (col, c)
/*++++++++++++++++
.PURPOSE  Increment the counts
.RETURNS  ---
-----------------*/
  size_t  col; 	/* IN: How many columns */
  int c;	/* IN: Character found  */
{
  int len; double v, atof();
    if (colsep) {
	/* if (colno <= MAXW) ccounts[colno*128 + c] += 1; */
	if ((c == colsep) || (c == '\n')) {	/* Separator found ! */
	    len = col - colstart[0] ;
	    if (colno >= nfields) expand_fields(nfields+128);
	    if (len > colwid(colno)) colwid(colno) = len ;
	    len = colstart1 < 0 ? 0  /* No non-blank char could be found */
		: 1 + colstart2 - colstart1;  /* First/Last non-blank */
	    if (len <= 0) { colpec(colno) |= F_NULL ; colnull(colno) += 1; }
	    if (colwnb(colno) < len) colwnb(colno) = len ;
	    if (len>0) {			/* Estimations */
	        if (colstart3 >= colstart1) {	/* # Decimals  */
	    	  if (colstart4 >= colstart3)	/* E-notation  */
	    	     len = colstart4-colstart3;
	          else len = colstart2-colstart3;
	          if (coldec(colno) < len) {
		      /*change_ccount("decimals", coldec(colno), len);*/
		      coldec(colno) = len;
		  }
	        }
	        if (colstart4 >= colstart1) {		/* E-notation  */
		  /*change_ccount("datatype", colpec(colno),
		     colpec(colno)|F_DBLE); */
	    	  colpec(colno) |= F_DBLE;
		}
	        if ((colstart5 >= colstart1) && ((colpec(colno)&F_CHAR)==0)) { 
		  int i;	/* non-numeric */
		    if (verbop) { 
			printf("\n#...col#%-3d  => non-numeric <", colno+1);
			for (i=0; i<icolcontent; i++) putchar(colcontent[i]);
			putchar('>');
			printf(llform[6], counts[_NL_]+1);
		    }
		    /* Verify possible "peculiar" NULLs */
	            if (icolcontent < sizeof(colcontent))
			colcontent[icolcontent] = 0;
	    	    i = tolower(colcontent[0]);
		    if (i == 'n') {	/* Check null */
			if (strcasecmp(colcontent, "nan") == 0) {
			    colpec(colno) |= F__nan; colnull(colno) += 1;
			}
			else if (strcasecmp(colcontent, "null") == 0) {
			    colpec(colno) |= F__nul; colnull(colno) += 1;
			}
			else colpec(colno) |= F_CHAR;
		    }
		    else if (strcasecmp(colcontent, "inf") == 0) {
			colpec(colno) |= F__inf; colnull(colno) += 1;
		    }
		    else if (issign(colcontent[0])) {
			for (i=0; colcontent[i] == '-'; i++) ;
			if ((i>1) && (colcontent[i] == 0)) {
			    colpec(colno) |= F__min; colnull(colno) += 1;
			}
			else if (strcasecmp(colcontent+1, "inf") == 0) {
			    colpec(colno) |= F__inf; colnull(colno) += 1;
			}
			else colpec(colno) |= F_CHAR;
		    }
		    else colpec(colno) |= F_CHAR;
		}
	    }
	    if ((occ > 1) && icolcontent 		/* Compute min/max */
	       && (icolcontent < sizeof(colcontent))) {
	         colcontent[icolcontent] = 0;
	         v = atof(colcontent);
	         if (v < colmin(colno)) colmin(colno) = v;
	         if (v > colmax(colno)) colmax(colno) = v;
	    }
	    colno++ ;
	    colstart[0] = col+1 ;
	    colstart1 = -1;	/* First non-blank char */
	    colstart2 = -1;	/* Last  non-blank char */
	    colstart3 = -1;	/* Position of decpoint */
	    colstart4 = -1;	/* Position of E 	*/
	    colstart5 = -1;	/* non-numeric value	*/
	    colstart6 = -1;	/* acceptable sign      */
	    icolcontent = 0;	/* To store the value	*/
	}
	else {
	    if (colno <= MAXW) ccounts[colno*128 + c] += 1;
	    if (!isspace(c)) {	/* Non-blank char found */
	        colstart2 = col ;
	        if (colstart1 < 0) colstart1 = col ;
		if (colstart5>0) ;	/* Already non-numeric, test useless */
		else if (isdigit(c)) ;
		else if ((c=='.') || (c==',')) {
		    if (colstart3 < 0) {	/* First decimal point */
		        colstart3 = col;
			colstart6 = col;	/* Sign not possible any more */
		    }
		    else colstart5 = col;	/* non-acceptable char */
		}
		else if (tolower(c) == 'e') {
		    if (colstart4 < 0) {	/* First Exponent      */
			colstart4 = col;
			colstart6 = -1;         /* Sign is possible */
		    }
		    else colstart5 = col;	/* non-acceptable char */
		}
		else if (issign(c)) {		/* Is sign possible ? */
		    if (colstart6<0) 		/* ... yes, it is */
			 colstart6 = col;
		    else colstart5 = col;
		}
		else colstart5 = col;
	        if (icolcontent < sizeof(colcontent))
		    colcontent[icolcontent++] = c;
	    }
	}
    }
    else if (col <= MAXW) ccounts[col*128 + c] += 1;
}

/*==================================================================*/
static int init()
/*++++++++++++++++
.PURPOSE  Define the fclass array
.RETURNS  Zero
-----------------*/
{
  int i;

    for (i=0; i<sizeof(fclass); i++) {
	if (isprint(i)) continue;
	switch(i) {
	  case '\n':
		fclass[i] = _NL_; continue;
	  case '\07': case '\b': case 033 /* Escape */:
	  case '\017': case '\016': /* Shift in + Out */
		fclass[i] = FC_ACTRL; continue;
	}
	if (isspace(i)) { fclass[i] = FC_AFILE; continue; } ;
	if (isascii(i)) { fclass[i] = FC_ABIN ; continue; } ;
	if (!oa) {
		if (isprint(ebcdic(i))){ fclass[i] = FC_EBCDIC; continue;} ;
	}
	fclass[i] = FC_BINARY;
    }
#ifdef DEBUG
	help();
	printf(".... init done ....");
#endif
    return(0);
}

static void show_files(tinfo)
/*++++++++++++++++
.PURPOSE  Display the Files
.RETURNS  ---
-----------------*/
  CATINFO *tinfo;	/* IN: Loaded ReadMe */
{
  FILINFO *f0, *f1 ;
  TABINFO *tbf;
  int i ;

    for (f0=tinfo->afinfo, f1=f0+tinfo->nfinfo; f0<f1; f0++) {
	printf("----File '%s'", f0->dfile) ;
	for (i=strlen(f0->dfile); i<12; i++) putchar(' ') ;
	printf(" %7dx%d,", f0->records, f0->reclen) ;
	i = f0->reclen ;
	if (i>0) while (i<1000) putchar(' '), i *= 10 ;
	if (f0->equinox) printf(" Equinox=%s,", f0->equinox) ;
	if (f0->epoch) printf(" Epoch=%s,", f0->epoch) ;
	if ((tbf=f0->tabform))
	    printf(" (Table#%d)", 1+(int)(tbf-tinfo->atinfo));
	else printf(" (No struct)") ;
	putchar('\n');
    }
}

static void show_tables(tinfo)
/*++++++++++++++++
.PURPOSE  Display table summary
.RETURNS  ---
-----------------*/
  CATINFO *tinfo;	/* IN: Loaded ReadMe */
{
  TABINFO *tbf; COLINFO *pf, *ef; char s;
  int i, j, count; short *ps;

    count = 0 ;
    for (i=tinfo->ntinfo, tbf=tinfo->atinfo; --i >= 0; tbf++) {
	printf("----Table#%d declared at line#%d: ", ++count, tbf->lineno);
	printf("%d cols", tbf->nfmt);
	if (tbf->posfmt[0] >= 0) {		/* There are RA+DE */
	    printf(", Equinox=%s, Epoch=%s\n",
		tbf->equinox ? tbf->equinox : "(?)",
		tbf->epoch   ? tbf->epoch   : "(?)");
	    printf("\t\t  position"); s='=';
	    for (ps=tbf->posfmt; *ps >= 0; ps++) {
	    	pf = tbf->afmt + *ps;
		printf("%c%s", s, pf->label);
		s = ',';
	    }
	    /* Show also the Proper Motions */
	    putchar(' '); s = '+' ;
	    for (pf = tbf->afmt, ef = pf + tbf->nfmt; pf < ef; pf++) {
		if (!(pf->flags & F_MPOS)) continue ;
		j = (pf - tbf->afmt) ;
		for (ps=tbf->posfmt; (*ps >= 0) && (*ps != j); ps++) ;
		if (*ps >= 0) continue ;
		printf("%c%s", s, pf->label);
		s = ',' ;
	    }
	}
	putchar('\n');
    }
}


/*==================================================================
		Save/Restore the "more" columns
 *==================================================================*/
static SAVEDmore *save_more(TABINFO *tform)
/*++++++++++++++++
.PURPOSE  Save the "more" columns, and return the structure with saved items
.RETURNS  NULL when all 'more' columns are null, or the saved addresses
-----------------*/
{
  COLINFO *pf, *pfe;
  SAVEDmore *new;
  int n, bytes;
    /* Count the number of elements */
    for (n=0, pf = tform->afmt, pfe = pf + tform->nfmt; pf<pfe; pf++) {
	if (pf->more) n++;
    }
    /* Allocate the full bytes only if at least one 'more' column exists */
    bytes = sizeof(SAVEDmore);
    if (n) bytes += tform->nfmt*sizeof(char *);
    new = (SAVEDmore*)malloc(bytes);
    new->table = tform;
    new->items = n;
    if (n) for (n=0, pf = tform->afmt; pf<pfe; pf++, n++) {
	new->saved_more[n] = pf->more;
	pf->more = (char *)0;
    }
    new->saved_more[n] = (char *)0;
    return(new);
}

static void restore_more(SAVEDmore *saved)
/*++++++++++++++++
.PURPOSE  Restore the "more" items from the TABINFO
.RETURNS  ---
-----------------*/
{
  COLINFO *pf, *pfe;
  int i;

    /* Count the number of elements */
    tform = saved->table;
    for (i=0, pf=tform->afmt, pfe=pf+tform->nfmt; pf<pfe; pf++) {
	pf->more = saved->items ? saved->saved_more[i++] : (char *)0;
    }
    memset(saved, 0, sizeof(SAVEDmore));
    free(saved);
}

/*==================================================================
		Dealing with Column Statistics
 *==================================================================*/

static int diffa(p1, p2)
/*++++++++++++++++
.PURPOSE  Comparison function to sort columns according to frequencies
.RETURNS  For qsort
-----------------*/
  char *p1, *p2;	/* IN: Characters to compare */
{
  int i1, i2;
    /* printf("....Compare %X (%d) and %X (%d)\n", *p1, pcounts[*p1],
    	*p2, pcounts[*p2]); */
    i1 = *p1 & 128 ? 128 : *p1;
    i2 = *p2 & 128 ? 128 : *p2;
    if (pcounts[i1] > pcounts[i2]) return(-1);
    if (pcounts[i2] > pcounts[i1]) return(1);
    return(i1-i2) ;
}

/*==================================================================
		Check against Format
 *==================================================================*/

static char *check_order(format, s)
/*++++++++++++++++
.PURPOSE  Check the Order
.RETURNS  Error message / NULL if OK
.MODIFIED 26-Jun-1996: In all cases, reset the previous value
-----------------*/
  COLINFO *format;	/* IN: Fortran Format */
  char *s;		/* IN: String to check*/
{
  double diff, atof();
  char *err;

    if (previous_value) free(previous_value), previous_value = (char *)0;
    err = (char *)0;
    if (!(format->elims & (F_SORT))) return((char *)0);

    if (isprint(*format->value)) {	/* No comparison with first value */
    	if (format->ff == 'A') diff = strcmp(s, format->value);
    	else diff = atof(s) - atof(format->value);
    	if ((diff == 0) &&  (!(format->elims & F_SRTE)))
	    err = err_bad_eq_;
    	else if ((diff < 0) && (!(format->elims & F_SRTD)))
	    err = err_not_inc;
    	else if ((diff > 0) && (format->elims & F_SRTD))
	    err = err_not_dec;
    }
    if (err) previous_value = strdup(format->value) ;
    strcpy(format->value, s);
    return(err);
}

static char *check_range(format, s)
/*++++++++++++++++
.PURPOSE  Check the String agains range
.RETURNS  Error message / NULL if OK
.REMARKS  From V7.35, format->more contains address of main COLINFO
		When s is "" or NULL ==> NULL value
-----------------*/
  COLINFO *format;	/* IN: Fortran Format */
  char *s;		/* IN: String to check*/
{
  char st, isint, isnul;
  double dval, *dlim=(double *)0, atof();
  int8 lval=0, *llim=(int8 *)0;

    isint = format->ff == 'I';
    isnul = 1;
    if (s) {
	while (*s == ' ') s++;
        if (*s) {
            if (isint) lval = atoll(s), dval = lval;
            else       dval = atof(s);
	    isnul = 0;
	}
    }

    /* Check against explicit NULL value */
    if ((format->elims&F_NULL) && (format->elims&F_REQD)) {
    	if (format->elims & F_DBLE) {
    	    dlim = (double *)format->lims;
    	    if (dlim[2] == dval) isnul = 1; /* V7.46 return((char *)0); */
    	} else {
    	    llim = (int8  *)format->lims;
    	    if (llim[2] == lval) isnul = 1; /* V7.46 return((char *)0); */
	}
    }
    if (isnul) { memset(&dval, -1, sizeof(dval)); lval = NULL8; }

    if (format->range && (!isnul)) switch(format->ff) {
      case 'A':		/* Should never be called */
        error("check_range with 'A' format should NEVER be called");
        break;
      case 'I':
        llim = (int8 *)format->range;
        llim[0] = MIN(llim[0], lval);
        llim[1] = MAX(llim[1], lval);
        break;
      default:
        dlim = (double *)format->range;
        dlim[0] = MIN(dlim[0], dval);
        dlim[1] = MAX(dlim[1], dval);
        break;
    }

    /* Store the value */
    if (format->elims & F_DBLE) {
    	dlim = (double *)format->lims;
	if (!dlim) {
	    dlim = (double *)malloc(4*sizeof(double));
	    format->lims = (char *)dlim;
	}
	dlim[3] = isint ? lval : dval;	/* Added V7.35 */
    }
    else {
	llim = (int8 *)format->lims;
	if (!llim) {
	    llim = (int8 *)malloc(4*sizeof(int8));
	    format->lims = (char *)llim;
	}
	llim[3] = isint ? lval : dval;   /* Added V7.35 */
    }
    if (isnul) return((char *)0);
    if (format->elims&F_MCMP) return((char *)0);	/* V7.36 (bounds) */
    if (!(format->elims & (F_IMIN|F_IMAX))) return((char *)0);

    if (format->elims & F_IMIN) {	/* Lower Bound */
	if (format->elims & F_DBLE) {
	    	st = (format->elims & F_XMIN) ? dval <= dlim[0] :dval < dlim[0];
        } else {
            if (isint)
            	st = (format->elims & F_XMIN) ? lval <= llim[0] :lval < llim[0];
	    else
            	st = (format->elims & F_XMIN) ? dval <= llim[0] :dval < llim[0];
        }
	if (st) return(err_too_small);
    }
    if (format->elims & F_IMAX) {	/* Upper Bound */
	if (format->elims & F_DBLE) {
	    	st = (format->elims & F_XMAX) ? dval >= dlim[1] :dval > dlim[1];
        } else {
            if (isint)
            	st = (format->elims & F_XMAX) ? lval >= llim[1] :lval > llim[1];
            else
            	st = (format->elims & F_XMAX) ? dval >= llim[1] :dval > llim[1];
        }
	if (st) return(err_too_large);
    }
    return((char *)0);
}

static char *check_special(pic, str)
/*++++++++++++++++
.PURPOSE  Check the components of NOT-NULL date / sexagesimal
.RETURNS  Error message
-----------------*/
  char *pic ;	/* IN: Units starting by '"' */
  char *str ;	/* IN: String to scan */
{
  static char err_msg[120] ;
#if 0
  char *p, *s ;
  int comp6[6] ;
  int i, j, c, ssep, sign;

    p = pic ; s = str ; i = 0 ;
    if (*p == '"') p++ ;
    comp6[0] = comp6[1] = comp6[2] = comp6[3] = comp6[4] = comp6[5] = 0 ;
    sign = 0 ;
    ssep = ispunct(p[1]) ;	/* ssep != 0 for h:m:s, i.e. separator */

    if (ssep) while (*s == ' ') s++ ;
    i = ssep ? 0 : 1 ;		/* Increment in picture for sign */
    if (*s == '-') s++, p += i, sign = 1 ;
    else if (*s == '+') s++, p += i ;

    while ( *s && *p && (*p != '"') ) {
	switch(c=*p) {
	  case 'Y': i=0 ; break ;
	  case 'M': i=1 ;
		if (p[1] == 'o') return ((char *)0) ;
			  break ;
	  case 'D': i=2 ; break ;
	  case 'H':
	  case 'h':
	  case 'd': i=3 ; break ;
	  case 'm': i=4 ;
		if (p[1] == 'o') return ((char *)0) ;
			  break ;
	  case 's': i=5 ; break ;
	  default:
	    if (*p == *s) { p++, s++ ; continue ; }
	    sprintf(err_msg, "****Invalid '%c' in unit %s <%%s>, column '%%s'",
		c, pic) ;
	    return(err_msg) ;
	}
	j = 0 ;
	if (comp6[i]) {
	    sprintf(err_msg, "****Invalid unit %s <%%s>, column '%%s'",
		pic) ;
	    return(err_msg) ;
	}
	if (ssep) {	/* Separator between numbers */
	    if (*s == ':') s++ ;
	    while (*s == ' ') s++ ;
	    while (isdigit(*s)) j = j*10 + (*(s++) - '0') ;
	    p++ ;
	    if (ispunct(*p) && (*s != '.')) {
		if (ispunct(*s)) s++ ;
		p++ ;
	    }
	}
	else {		/* e.g. YYYYMMDD */
	    while ((*p == c) && isdigit(*s)) j = j*10 + (*(s++) - '0'), p++ ;
	    if (*p == c)  {
		sprintf(err_msg, "****Non-digit for %s <%%s>, column '%%s'",
		   pic);
		return(err_msg) ;
	    }
	}
	comp6[i] = j ;
	if ((*s == '.') && (*p != *s)) break ;
    }
    if (*p == '"') p++ ;

    /* Verify Validity of Month, etc */
    if (comp6[1] > 12) {
	sprintf(err_msg, "++++Bad mon (%d) for %s <%%s>, column '%%s'",
	    comp6[1], pic) ;
	return(err_msg) ;
    }
    if (comp6[2] > 31) {
	sprintf(err_msg, "++++Bad day (%d) for %s <%%s>, column '%%s'",
	comp6[2], pic) ;
	return(err_msg) ;
    }
    if (comp6[4] >= 60) {
	sprintf(err_msg, "++++Bad min (%d) for %s <%%s>, column '%%s'",
	comp6[4], pic) ;
	return(err_msg) ;
    }
    if (comp6[5] >= 60) {
	sprintf(err_msg, "++++Bad sec (%d) for %s <%%s>, column '%%s'",
	comp6[4], pic) ;
	return(err_msg) ;
    }
    if (*s == '.') {
	for (++s; isdigit(*s); s++) ;
    	while (*s == ' ') s++ ;
    }
    if ((!*s) || (*s == '\n')) return ((char *)0) ;
    sprintf(err_msg, "****Invalid char '%s' <%%s>, column '%%s'",
	edchar(*s)) ;
    return(err_msg) ;
#else
    double value; char *msg ;
    msg = unit_get(pic, str, &value) ;
    if (!strchr(msg, '?')) return((char *)0) ;
    sprintf(err_msg, "++++%s in conversion %s on '%s'", msg, pic, str) ;
    return(msg) ;
#endif
}

static int acheck(format, s)
/*++++++++++++++++
.PURPOSE  Check the String against the Format
.RETURNS  0 = OK / 1 = error
-----------------*/
  COLINFO *format;	/* IN: Fortran Format */
  char *s;		/* IN: String to check*/
{
  char c, *p, *b, *pe=(char *)0, *err, *label, *anull, edichar, all_dashes;
  int8 *alim;
  int nd;

    p = s; c = all_dashes = 0; err = (char *)0;
    edichar = 0;	/* This flag indicates we've to edit a CHAR in case
    				of error (default is STRING) */
    if (!format) {	/* NULL for blanks ONLY */
	if (!issep(*p)) err = err_non_bla;
	label = "blank";
	edichar = 1;
	goto Retour;
    }
    label = format->label;
    if (format->len == 0) {	/* Too long line... */
	while (*p != '\n') {
	   if (!isspace(*p)) { err = err_too_lon; break; }
	   p++;
	}
	goto Retour;
    }
    for (pe = p, nd = format->len; --nd >= 0; pe++) {
	if (*pe == '\n') break;
    }
    /* Remove the trailing spaces (V7.51)*/
    if (*pe == '\n') { while (isspace(*pe) && (pe>=p)) --pe; pe++; }
    c = *pe; *pe = 0;

    while(*p == ' ') p++;
    if ((*p == '\n') || !*p) { 		/* Accept blank sign ! */
	if ((format->elims & F_REQD) && (!(format->flags&F_SIGN)))
	    err = err_bad_nul;
	if (format->ff != 'A')
	    check_range(format, "");	/* Required to set value */
	goto Retour;
    }

    /* Verify possible NULL values */
    anull = "";
    if ((format->elims && format->lims) && (format->elims&F_CHAR)) {
	/* NULL given as byte, e.g. * or - */
	anull = format->lims
	   + 16 /*2*(format->elims & F_DBLE ? sizeof(double) : sizeof(int8))*/;
    }
    if (!anull[0]) anull = " ";

    /* all_dashes indicate an acceptable NULL value in numeric format */
    for (b = p; (*b == '-') || (*b == ' ') || (*b == *anull) ; b++) ;
    all_dashes = (! *b) || (*b == '\n');

    if (format->flags & (F_DATE|F_TIME|F_ANGL)) {
	/* Take care of Special ?=0 for DATES */
	if ((format->ff == 'I') && (format->elims&F_NULL)) {
	    alim = (int8 *)(format->lims) ;
	    while (isdigit(*p)) p++ ;
	    if (atoll(p) == alim[2]) goto Retour;
	}
	err = check_special(format->unit, s) ;
	if (!err) p = pe ;
	goto Retour;
    }
    /* printf("Format: %s\n", fmt_ed(format)); */
    switch(format->ff) {
    case 'A':
    	edichar = 1;
	if (format->elims & F_ATRT) {	/* We've to check against allowed */
	    while (format->lims[*(unsigned char *)p]) p++;
#if 1  /*O_BINARY	-- MS-DOS: new lines are \r\n */
  	    if(*p == '\r') p++;
#endif
	    if (p != pe) {
		err = err_bad_car;
		break;
	    }
	}
	if (fop&FOP_ra) for (p = s; *p; p++) {
	    format->range[*(unsigned char *)p] = 1;
	}
    	while (isascii(*p) && isprint(*p)) p++;
    	if (format->elims & F_SORT) {	int diff; 	      /* Check Order */
    	    if (isprint(*format->value)) { /* No comparison with first value */
		diff = strncmp(s, format->value, format->len);
    		if ((diff == 0) && (!(format->elims & F_SRTE)))
	    	    err = err_bad_eq_;
    		else if ((diff < 0) && (!(format->elims & F_SRTD)))
	    	    err = err_not_inc;
    		else if ((diff > 0) && (format->elims & F_SRTD))
	    	    err = err_not_dec;
	    }
	    strncpy(format->value, s,  format->len);
	    if (err) goto Retour;
    	}
	if (*p == '\n') goto RetourOK;
    	goto Retour;
    case 'I':
	err = check_range(format, all_dashes ? "" : p);
	if (all_dashes) {
	    if (format->elims & F_REQD) err = err_bad_val;
	    else p = pe;    /* pe was computed before = end of input column */
	    goto Retour;
	}
    	if (err) break;
    	if (issign(*p)) p++;
    	for(nd=0; isdigit(*p); nd++) p++;
	if (p != pe)  err = err_non_dig, edichar=1;
	else if (nd == 0)
	    p = s,
	    err = err_mis_dig;
	break;
    case 'F':
    	err = check_range(format, all_dashes ? "" : p);
	if (all_dashes) {
	    if (format->elims & F_REQD) err = err_bad_val;
	    else p = pe;    /* pe was computed before = end of input column */
	    goto Retour;
	}
    	if (err) break;
	if (s[(format->odp)&0xff] != '.') {
	    p = s + format->odp;
	    err = err_bad_dec;
	    edichar = 1;
	    goto Retour;
    	}
	nd = 0;
    	if (issign(*p)) p++;
    	while(isdigit(*p)) p++, nd++;
    	if (*p == '.') p++;
    	while(isdigit(*p)) p++, nd++;
	if (nd == 0)
	    p = s,
	    err = err_mis_dig;
    	break;
    case 'E': case 'D':
    	err = check_range(format, all_dashes ? "" : p);
	if (all_dashes) {
	    if (format->elims & F_REQD) err = err_bad_val;
	    else p = pe;    /* pe was computed before = end of input column */
	    goto Retour;
	}
    	if (err) break;
    	if (issign(*p)) p++;
	nd = 0;
    	while(isdigit(*p)) p++, nd++;
    	if (*p == '.') p++;
    	while(isdigit(*p)) p++, nd++;
    	if(isexpo(*p)) {
    	    p++;
    	    if (issign(*p)) p++;
    	    while(isdigit(*p)) p++;
	}
	if (nd == 0)
	    p = s,
	    err = err_mis_dig;
	break;
    default:
    	err =  err_bad_fmt, p = &format->ff;
	break;
    }
			/* Number: have to check the Order */
    if ((!err) && (format->elims & F_SORT))
	if ((err = check_order(format, s))) p = s;
    Retour:
    if (!err) {
  	if (c) {
  	    while(*p == ' ') p++;
#if O_BINARY		/* MS-DOS: new lines are \r\n */
  	    if(*p == '\r') p++;
#endif
  	    if ((*p) && (*p != '\n')) err = err_bad_car;
  	}
    }
    if (err) { char *px=(char *)0, x; FILE *ferr;
	ferr = fop&FOP_2 ? stderr : stdout;
	/* V6.5: Edit the column number */
  	if (!error_1) { putchar('\n'); error_1 = 1; }
	if ((fmtno>0) && (px = strstr(err, "olumn '%s"))) {
	    if (strchr(err, '%') > px) px = (char *)0;
	    else px += 2;	/* to replace "column" by "col#ccc" */
	}
	/* Replace column by col#ccc, and add blanks */
	if (px) {
	    x = *px; *px = 0;
	    fprintf(ferr, err, edichar ? edchar(*p) : p);
	    *px = x;
	    px += strloc(px, '\''); if (*px) px++;
	    px += strloc(px, '\''); if (*px) px++;
	    fprintf(ferr, "#%03d ", fmtno);
	    nd = 10 - strlen(label);	/* Blanks to add */
	    if (nd<0) nd=0;
	    if ((label[1] != '_') && (nd>=2)) {
		fprintf(ferr, "  ");
		nd -= 2;
	    }
	    fprintf(ferr, "'%s'", label);
	    while (--nd>=0) fputc(' ', ferr);
	    fprintf(ferr, "%s", px);
	}
	else fprintf(ferr, err, edichar ? edchar(*p) : p, label) ;
    	if (previous_value) fprintf(ferr, " <%s>", previous_value);
	fprintf(ferr, ", line #%d.%d ", lineno+1, lsize+1 + (int)(p-s));
	fprintf(ferr, "(%s)\n", fmt_ed(format));
    }
    RetourOK:
    if (c) *pe = c;
    return(err ? 1 : 0);
}

static int rcheck(COLINFO **cols)
/*++++++++++++++++
.PURPOSE  Verify dependencies (upper/lower bounds)
.RETURNS  Number of errors
-----------------*/
{
  static FILE *ferr;
  static double nullf = -1;
  COLINFO *pf, **acol, *rf;
  double value, rvalue;
  double *alimf;
  int8   *alim;
  char *err;
  int nerr=0;

    if (nullf == -1) {	/* Initialize */
	memset(&nullf, -1, sizeof(nullf));
	ferr = fop&FOP_2 ? stderr : stdout;
    }
    for (acol=cols; *acol; acol++) {
	pf = *acol; rf = (COLINFO *)pf->more;
	if (!rf) continue;	/* Not normal!!! */
	if (pf->elims&F_CHAR) continue;
	if (rf->elims&F_CHAR) continue;
	/* Get referent value */
	if (rf->elims & F_DBLE) {
	    alimf = (double *)rf->lims;
	    rvalue = alimf[3];
	}
	else {
	    alim = (int8 *)rf->lims;
	    rvalue = alim[3];
	}
	/* Get current value */
	if (pf->elims & F_DBLE) {
	    alimf = (double *)pf->lims;
	    value = alimf[3];
	    alim = 0;
	}
	else {
	    alim = (int8 *)pf->lims;
	    value = alim[3]; if (alim[3] == NULL8) value = 0./0.; /* V7.42 */
	    alimf = 0;
	}
	/* Verify value compared to referent */
	err = 0;
	if (pf->elims&F_IMIN) {
	    if (alimf && (memcmp(&alimf[0],&nullf,8)==0) && (value<rvalue))
		err = errf_too_small;
	    else if (alim && (alim[0] == NULL8) && (value<rvalue))
		err = errf_too_small;
	}
	if (pf->elims&F_IMAX) {
	    if (alimf && (memcmp(&alimf[1],&nullf,8)==0) && (value>rvalue))
		err = errf_too_large;
	    else if (alim && (alim[1] == NULL8) && (value>rvalue))
		err = errf_too_large;
	}
	if (err) { int st;
  	    if (!error_1) { putchar('\n'); error_1 = 1; }
	    st = fprintf(ferr, err, value, rvalue, pf->label);
	    fputc(',', ferr);
	    while (st<60) { fputc(' ', ferr); st++; }
	    fprintf(ferr, " line #%d\n", lineno+1);
	    nerr++;
	}
    }
    return(nerr);
}


/*==================================================================
		Checking the References
 *==================================================================*/
static int diffr(a, b)
/*++++++++++++++++
.PURPOSE  Compare two strings (but ignore leading spaces)
.RETURNS  a - b
-----------------*/
  char *a, *b ;
{
    while (*a == ' ') a++ ;
    while (*b == ' ') b++ ;
    while (*a && (*a == *b)) a++, b++ ;
    while (*a == ' ') a++ ;
    while (*b == ' ') b++ ;
    return(*a - *b) ;
}

static int diffi(a, b)
/*++++++++++++++++
.PURPOSE  Compare two numbers
.RETURNS  a - b
-----------------*/
  int *a, *b ;
{
    return(*a - *b) ;
}

static int load_refs(filinfo)
/*++++++++++++++++
.PURPOSE  Load the References
.RETURNS  Number of references loaded in references
-----------------*/
  FILINFO *filinfo;	/* IN: The "refs.dat" file */
{
  TABINFO *tform ;
  COLINFO *pf ;
  FILE *fp ;
  char *p, *aref, *eref, *pref, *record ;
  int *iref, lrec, i, n, blank_ref, blank_lines ;
  int dupref = 0;

    fp = fizopen(filinfo->dfile) ;
    if (!fp)  perror(filinfo->dfile), terminate("Problem with refs.dat");

    /* Look in the code of references */
    tform = filinfo->tabform ;
    if (!tform) {
	fflush(stdout) ;
	fprintf(stderr,
	"\n++++No Byte-by-Byte Description for reference file: %s\n",
	filinfo->dfile) ;
	return(0) ;
    }
    pf  = tform->afmt;
    record = malloc(lrec = filinfo->reclen + 80) ;
    references.nr = filinfo->records ;
    if (pf->ff == 'I') {
	references.rint = 1 ;
	references.lr = sizeof(int) ;
	references.diff = (SORT_FCT)diffi ;
    }
    else {
	references.rint = 0 ;
	references.lr = pf->len +1 ;
	references.diff = (SORT_FCT)diffr;
    }

    if (references.refs) free(references.refs) ;
    references.refs = malloc((references.nr+1) * references.lr) ;
    references.ref  = references.refs + references.nr*references.lr ;
    aref = references.refs ;
    iref = (int *)aref ;

    /* Read Completely the refs.dat file */
    pref = record + pf->b0 ;
    eref = pref + pf->len ;
    blank_ref = blank_lines = 0 ;
    for (n=0; fgets(record, lrec, fp); ) {
	*eref = 0 ;
	for (p = pref; isspace(*p); p++) ;
	if (p >= eref) { blank_ref++; continue ; }
	if (*p == 0) { blank_lines++; continue ; }	/* Blank line */
	/* Compare with previous */
	if (references.rint) {
	    i = atoi(pref) ;
	    if (n && (i == iref[-1])) continue ;
	    *(iref++) = i ;
	}
	else {
	    if (n && (strcmp(pref, aref - references.lr) == 0))
		continue ;
	    strcpy(aref, pref) ;
	    aref += references.lr ;
	}
	if (n >= references.nr) {
	    sprintf(record, "More than %d references in file: %s",
		references.nr, filinfo->dfile) ;
	    fizclose(fp) ;
	    terminate(record) ;
	}
	n++ ;
    }
    fizclose(fp) ;
    references.nr = n ;	/* Actualize the number of references */

    if (blank_lines) printf(" (++%d blank lines+\a+) ", blank_lines) ;
    if (blank_ref) printf("\n++++ %d blank references ??\n", blank_ref) ;

    /* Sort the References */
    qsort(references.refs, references.nr, references.lr, references.diff) ;

    /* Check if no reference is duplicated */
    i = references.nr ;
    if (references.rint) for (iref = (int *)references.refs; --i > 0; iref++) {
	if (iref[1] != iref[0]) continue ;
	 dupref++;
	 printf("\n++++Duplicated reference: %d", *iref) ;
    }
    else for (aref = references.refs; --i > 0; aref += references.lr) {
	if (strcmp(aref, aref + references.lr)) continue ;
	 dupref++;
	 printf("\n++++Duplicated reference: %s", aref) ;
    }
    if (dupref) printf("\n    ");

    free(record) ;
    return(references.nr) ;
}

static int check_refs(filinfo)
/*++++++++++++++++
.PURPOSE  Check the references vs refs.dat
.RETURNS  Number of errors / -1 (not concerned)
.REMARKS  Set the static count_nullrefs
-----------------*/
  FILINFO *filinfo;	/* IN: The "refs.dat" file */
{
  COLINFO *pf, *pfe;
  TABINFO *tform ;
  FILE *fp ;
  char *aref, *eref, *xref, *record, x, y;
  int *iref=(int *)0;
  int8 *llim;
  int len, lrec, i, nerr, nrefcol, isnullref, nrec, dref;

    count_nullrefs = 0;
    count_refcols = 0;
    nrefcol = nerr = nrec = 0 ;
    tform = filinfo->tabform ;

    /* Are there columns with references ? (v6.4) */
    for (pf = tform->afmt, pfe = pf + tform->nfmt; pf < pfe; pf++) {
	if (pf->flags & F_USER) fprintf(stderr,
	    "++++Flagged (F_USER) column will be reset: table=%s, col=%s\n",
	    filinfo->dfile, pf->label);
	if (tolower(pf->label[0]) != 'r') continue;
	if (pf->label[1] == '_') ;
	else if (strmatch("[Rr]ef[s()]*", pf->label)) ;
	else continue;
	pf->flags |= F_USER;
	nrefcol++;
    }
    if (nrefcol == 0) 				/* NO REFERENCE COLUMN */
	return(-1);

    fp = fizopen(filinfo->dfile) ;
    if (!fp)  perror(filinfo->dfile),
	terminate("Problem with data file");
    record = malloc(lrec = filinfo->reclen + 80) ;

    /* v6.4: use the F_USER bit to mark columns having a reference */
    while (fgets(record, lrec, fp)) {
	len = strloc(record, '\n') ;
        for (pf = tform->afmt, pfe = pf + tform->nfmt; pf < pfe; pf++) {
	    if (pf->flags&F_USER) ; else continue;
	    if (len <= pf->b0) continue ;
	    aref = record + pf->b0 ;
	    eref = aref + pf->rep*pf->len ;	/* v6.4 */
	    if (eref > (record+len)) eref = record+len ;
	    x = *eref; *eref = 0;
            llim = (int8 *)0; isnullref = 0;
	    while (aref < eref) {
	        if (*aref == ',') { aref++; continue; }
	        if (*aref == ' ') { aref++; continue; }
	        if (*aref == '(') { aref++; continue; }
	        if (*aref == ')') { aref++; continue; }
	        if (*aref == '-') { aref++; continue; }
		dref =  pf->rep > 1 ? 	/* v6.4: accept array of references */
		  pf->len :
		  strloc(aref, ',');	/* v6.4 */
	        if (references.rint) {
		    if (!isdigit(*aref)) {
			printf("\n++++Reference in col. '%s' non-numeric: ",
			  pf->label);
			fwrite(aref, 1, eref-aref, stdout);
			aref = eref;
			continue;
		    }
		    iref = (int *)references.ref ;
		    y = aref[dref]; aref[dref] = 0;
		    *iref = atoi(aref) ;
		    aref[dref] = y;
	            if (pf->elims&F_NULL) {	/* Can be NULL reference */
		        llim = (int8 *)(pf->lims);
		        isnullref = *iref == llim[2];
	            }
	        }
	        else {
		    xref = aref + dref;
		    if (xref > eref) xref = eref ;
		    i = xref - aref ;
		    if (i >= references.lr) i =  references.lr - 1 ;
		    strncpy(references.ref, aref, i) ;
		    while (i < references.lr) references.ref[i++] = ' ' ;
		    references.ref[references.lr - 1] = 0 ;
	        }
		if (isnullref) count_nullrefs++;	/* NULL reference */
	        else if(!bsearch(references.ref, references.refs, references.nr,
		    references.lr, references.diff)) {
	            /* Reference not found */
	            printf("\n++++Reference in col. '%s' not found: ",
			pf->label) ;
	            if (references.rint) printf("%d", *iref) ;
	            else printf("%s", references.ref) ;
		    nerr++ ;
		}
		aref += dref;
	    }
	    *eref = x;
	}
	nrec++;
    }

    /* Reset the F_USER bit to zero */
    for (pf = tform->afmt, pfe = pf + tform->nfmt; pf < pfe; pf++)
	pf->flags &= ~F_USER;

    /* Return: free allocated elements, set the results */
    count_refcols = nrefcol;
    free(record) ;
    fizclose(fp) ;
    return(nerr) ;
}

/*==================================================================
		General Checking
 *==================================================================*/

static int examine(buf, size)
/*++++++++++++++++
.PURPOSE  Examine Buffer, to derive a guess
.RETURNS  Data-file type
.REMARKS  Is not always exact...
-----------------*/
  char *buf;      /* IN: Data to examine */
  int  size;      /* IN: Size of buffer  */
{
  char *p, *pe ;
  int  i, ftyp, nebc;
  static char *fitsmark[] = {
	  "SIMPLE  = ", "XTENSION= "
  };

    buf[size] = 0;

    ftyp = FC_ABULK;
    for (i=0; i < ITEMS(fitsmark); i++) {
  	if (strncmp(buf, fitsmark[i], strlen(fitsmark[i])) == 0)
	   ftyp = FC_FITS;
    }
    pe = buf + (ftyp == FC_FITS ? 2880 : size);
    bzero (counts, sizeof(counts)); nebc = 0;

#ifdef DEBUG
    printf("....Initial ftype = %s\n", ftype(ftyp));
#endif
    for (p = buf; p < pe; p++) {
	i = fclass[*p & 0xff];
	if (oa && (i == FC_BINARY || i == FC_ABIN)) i = FC_ABIN;
	counts[i] += 1;
	if (ebcdic(*p)) nebc++;
    }
    if (ftyp == FC_FITS) {
	/* Assume pure FITS if no newline found here */
	return (nls ? FC_FITS_NL : FC_FITS);
    }

    if (nebc != size) 	counts[FC_EBCDIC] = 0;

#ifdef DEBUG
    for (i = _NL_; --i >= FC_ABIN; )
	printf(".... counts[%s]=%d\n", ftype(i), counts[i]);
#endif

    for (i = _NL_; --i >= FC_ABIN; )
	if (counts[i]) return(i);

	/* We've an ASCII file. Check for newlines */
    if (nls == 0) {
	if ( counts[FC_AFILE] || counts[FC_ACTRL]) counts[FC_ABIN] = 1;
    }
    else  counts[FC_AFILE] = 1;

    for (i = FC_ABIN; i >= 0; i--)
	if (counts[i])  return(i);

    return(ftyp);        /* Never reached, but... */
}

/*==================================================================*/
static int dfile(name)
/*++++++++++++++++
.PURPOSE  Examine Directory, and derive statistics
.RETURNS  Data-file type
.REMARKS  Nothing in MS-DOS...
-----------------*/
  char *name ; 	/* IN: Directory name  	*/
{
#if O_BINARY 	/* MS-DOS */
    printf(": . x . : . :") ;
#else		/* If problem with dirent.h, Try to #unset HAVE_DIRENT */
  struct dirent *dir;
  DIR *dirp ;
  struct stat buf ;
  int4 nb ; int nf ;

    if (!(dirp = opendir(name))) { perror(name) ; return(-1) ; }
    if (chdir(name) < 0) { perror(name) ; return(-1) ; }
    nb = nf = 0 ;
    while ((dir = readdir(dirp))) {
	if (dir->d_name[0] == '.') continue ;	/* Forget hidden files */
	nf++ ;
	if (stat(dir->d_name, &buf) < 0) { perror(dir->d_name); continue ; }
	/* Size in number of blocks */
	nb += 1 + (buf.st_size>>9) ;	/* 1 block for DIR entry */
	if (buf.st_size&511) nb += 1;
    }
    if (chdir("..") < 0) { perror("..") ; return(-1) ; }
    closedir(dirp) ;
    printf(": %d x . : %dK :", nf, (nb+1)/2 ) ;
#endif
    return(FC_DIRECTORY) ;
}

static int efile(infile, ocount)
/*++++++++++++++++
.PURPOSE  Examine Opened File, and derive statistics
.RETURNS  Data-file type
.REMARKS  Statistics are also performed
-----------------*/
  FILE *infile;	/* IN: Opened file number */
  int ocount;	/* IN: -cc option        */
{
  static char list_numeric[] = "0123456789.,+-einfaul";
  static char acceptable_numeric[128];
  static char empty_line[] = "\n" ;
  static char fmt_letter[] = "FAIA";
  static char head_exp[2]  = "\0";
  char *tschema = table_schema;
  int ftyp = 0;  	/* File Type */
  int in_head = 0;	/* boolean */
  int pcprint = 1;	/* Verbose mode: print 1/pcprint message */
  int non_ascii, fixed, c, i, j, k, m, ab, nw;
  size_t size, n=0;
  int8  nb, total;
  int8  sums[129]; /* sums = frequency of chars */
  int4  parseps = 0;	/* Empty lines	*/
  int4  heading = 0;	/* Head  lines	*/
  FILINFO *filinfo;
  COLINFO *pf, *pfe, **cols2check=0;	/* cols2check: for upper/lower bounds */
  SAVEDmore *saved_more = (SAVEDmore *)0;
  static char buffer[BUFb+1];
  char fits_dim[256];
  char *ps, *pe, *p, *b, *colnames=0, x;
  int hlines; char *hexp;
  int ncols=0;

    /* Initialize */
    if (acceptable_numeric['0'] == 0) for (i=0; list_numeric[i]; i++) {
	x = list_numeric[i];
	acceptable_numeric[(int)x] = 1;
	if (isalpha(x)) acceptable_numeric[toupper(x)] = 1;
    }
    bzero (sums, sizeof(sums));
    hlines = headlines; hexp = headexp;
    fits_dim[0] = 0;

    if (!tschema) tschema = "[-f](stdin)";

    /* Read the First Block (2880 bytes) and Examine it */

    size = k = m = 0;
    while (size < 2880) {
	n = fread(buffer+size, 1, (sizeof(buffer)-1)-size, infile);
	if (ferror(infile)) { errors++, perror(tschema); break; }
	if (n == 0)     break;
	size += n;
    }

    /* if (errors)   terminate((char *)0); */

    if (ftyp == FC_BINARY && tinfo)   /* A format is given; can't be binary! */
	ftyp = FC_ABIN;
    ftyp = examine(buffer, size);       /* Examine */

    printf(" %s ", ftype(ftyp));

    if (ocount)   ofull = 1;
    if (ftyp == FC_FITS) {
	ocount = 0;    		/* Don't do col stats! */
	/* Try to find the FITS dimensions */
	for (ps=buffer, pe=buffer+n; ps<pe; ps += 80) {
	    if (strncmp(ps, "NAXIS ", 6) == 0) break;
	}
	if (ps < pe) {
	    for (p = ps + strloc(ps, '=')+1; *p == ' '; p++) ;
	    m = atoi(p);	/* Number if axises */
	    ps += 80;
	    b = fits_dim;
	    for (i=0; (i<m) && (ps<pe); ps += 80) {
		if (strncmp(ps, "NAXIS", 5) !=0) break;
		if (i) *(b++) = 'x';
		i++;
		if (i != (ps[5]-'0')) {
		    fprintf(stderr, "****Bad NAXIS%d in FITS file\n", i);
		    break;
		}
		else {
		    for (p = ps + strloc(ps, '=')+1; *p == ' '; p++) ;
		    while(isdigit(*p)) *(b++) = *(p++);
		}
	    }
	    *b = 0;
	}
    }

    if (!ofull) {
	if (fits_dim[0]) printf("[%s] ", fits_dim);
	return (0);
    }

	/* Examine Completely the File Characteristics */

    fflush(stdout);			/* Added V6.73 */
    n = size;
    if (headbuf) { free(headbuf); headbuf = (char *)0; lheadbuf = 0; }
    if (has_head) {				/* Keep headlines */
	lheadbuf = n;
	headbuf = malloc(lheadbuf+1);
	memcpy(headbuf, buffer, lheadbuf);
	headbuf[lheadbuf] = 0;
    }
    bzero(counts, sizeof(counts));
    if (fieldstat) bzero(fieldstat, nfields*sizeof(FIELDSTAT));
    else expand_fields(128);
    for (i=0; i<nfields; i++) {
	colmin(i) = 9.e99, colmax(i) = -9.e99;
    }
    non_ascii = size = reclen = lsize = nb = lineno = colno = fmtno
	      = colstart[0] = total = colstart1 = 0;
    colstart2 = colstart3 = colstart4 = colstart5 = colstart6 = -1;
    fixed = ( (ftyp == FC_ABULK) || (ftyp == FC_EBCDIC)
	  ||  (ftyp == FC_FITS)  || (ftyp == FC_BINARY) );
    if (colsep) fixed = 0;	/* TSV data can't have a fixed length */
    if (fixed)
	reclen = ftyp == FC_FITS ? 2880 : lcount;
    if (ocount)   memory(reclen ? reclen : lcount);

    if (tinfo) {	/* -f option: set up COLINFO pointers */
	filinfo = cad_addfile(tinfo, tschema);
	if (!filinfo) {	/* This can happen when ReadMe is to be analyzed */
	    fprintf(stderr, "++++File '%s' can't be analyzed by '%s'\n",
		tschema, tinfo->ffile);
	    return(ftyp);
	}
	tform = filinfo->tabform;
	if (!tform) {
	    fprintf(stderr, "\n**** File '%s' not described in '%s' ****\n",
	    	tschema, tinfo->ffile);
	    terminate((char *)0);
    	}
	tbf_reset(tform);
    	pf  = tform->afmt;
    	pfe = pf + tform->nfmt;
	/* V6.7: accept headlines, expressed in the Byte-by-byte Description */
	if (tform->headlines || tform->headchar) {
	   hlines = tform->headlines;
	   hexp = head_exp;
	   head_exp[0] = tform->headchar;
	}
    }
    else filinfo = (FILINFO *)0, tform = (TABINFO *)0, pf = pfe = (COLINFO *)0;

    if (verbop)
	printf(" (Number of Bytes Read:\n"), fflush(stdout), error_1=1;

    ps = buffer;	error_1 = fop&FOP_2;
    in_head = (hlines>0) || (hexp != 0) /*hexp ? 1 : 0*/;

    /* V7.35: lower/upper bounds ==> used the "more" as COLINFO pointer
     * to the parent column */
    if (tform) { int n = 0;	/* How many columns with indirect check */
        COLINFO *pcol;
	saved_more = save_more(tform);
	for (; pf<pfe; pf++) {
	    if (pf->label[0] && (pf->label[1] == '_')) {
		if ((pcol = tbf_findlabel(tform, pf->label+2))) {
		    pf->more = (char *)pcol;
		    if (pf->elims&F_MCMP) n++;
		}
	    }
	    else pf->more =  (char *)0;
	}
	if (n) { /* Specify 2nd pass for verification of lower/upper bounds */
	    cols2check = (COLINFO **)malloc((n+1)*sizeof(COLINFO *));
	    for (n=0, pf=tform->afmt; pf<pfe; pf++) {
		pcol = (COLINFO *)pf->more;
		if (!pcol) continue;
		if (pf->elims&F_MCMP) cols2check[n++] = pf;
	    }
	    cols2check[n] = 0;	/* Terminate the list of cols to check */
	}
	pf  = tform->afmt;
    }

    while(n > 0) {
  	pe = ps + n; ps = buffer; *pe = 0;
	nb++;           /* Number of Buffers Read */
	if (verbop)   { /* Write out where we're */
	    /* We show every buffer at the beginning, and then less and
	     * less frequently (1/1000 after millionth) */
	    static int ilim = 0, m=0;
	    static char mc[] = " kMG";	/* Edit a fraction of the number */
	    static int8 mf[] = { 1, 100, 10000, 1000000 };
	    static int8 mlim[] = { E8, 100*E8, 10000*E8,1000000*E8  };
	    static int blim[] = { 
		10, 50, 200, 1000, 5000, 20000, 100000, 500000, 1000000, 0 };
	    static int plim[] = {  
		2,  5,  10,   20,   50,   100,    200,     500,    1000, 0 };
	    if ((nb%pcprint) == 0) {
		if ((total+n)>= mlim[m]) m++;
		if (m==0) printf (llform[0], total+n);
		else      printf (llform[7], (total+n)/mf[m], mc[m]);
		fflush(stdout);
		if (nb == blim[ilim]) { 
		    ilim++;
		    /*printf(" (pc[%d]=%d)", ilim, plim[ilim]);*/
		    if (plim[ilim]) pcprint=plim[ilim];
		}
	    }
	    error_1 = fop&FOP_2; /* Tells that a '\n' should be issued before
				    the next error message (only when error
				    messages are issued on stdout)          */
	}
	for (p = ps; p < pe; p++, total++) {
	    if (in_head) {
		if (*p == '\n') {
		    heading++;		/* One headline more */
		    colno=0; lineno++;
		    continue;
		}
		if (colno == 0) {	/* First byte in line      */
		    if (!hexp) 		/* Header defined by count */
		         in_head = lineno < hlines;
		    else in_head = *hexp == *p;
		}
		if (in_head) { colno++; continue; }
	    }
	    else if (hexp && *hexp && (colno == 0)) {
					/* V7.2: may be comment */
		in_head = *p == *hexp;
		if (in_head) { colno++; continue; }
	    }
	    k = fclass[*p &  0xff];
	    counts[k] += 1;
	    if (fixed && (!ocount))	/* e.g. FITS */
		continue;
	    if (oa && (k == FC_BINARY || k == FC_ABIN)) {
		if (non_ascii == 0) putchar('\n');
		non_ascii++;
		printf (llform[1],
		    *p & 0xff, 1 + counts[_NL_], lsize, total);
		for (x=1; isprint(p[x&0xff]) && x<30; x++) putchar(p[x&0xff]);
		putchar('\n');
	    }
	    if ((*p == '\n') && (lsize == 0)) {	/* V6.51: count empty lines */
		parseps++;
		lineno++;
		continue;
	    }
	    if (tform) {	/* Format */
		if (*p == '\n') {	/* Verify last fields */
		    /* Empty Line test -- Mod. v6.24 2003 */
		    if ((pf == tform->afmt) && (lsize <= pf->b0)) {
			parseps++;
			lineno++;
			continue;
		    }
		    if (lsize > pf->b0) pf++ ;
		    while (pf < pfe) {
			fmtno = 1 + (pf-tform->afmt);
			errors += acheck(pf++, empty_line);
		    }
		    pf = tform->afmt;
		    /* V7.35: Verify dependencies */
		    if (cols2check) errors += rcheck(cols2check);
		}
		else if (pf < pfe) {
		    if (lsize > pf->b1)  pf++;
		    fmtno = 1 + (pf-tform->afmt);
		    if (pf >= pfe) errors += acheck(&end_format, p);
		    else if (lsize < pf->b0) errors += acheck((COLINFO *)0, p);
		    else if (((lsize-pf->b0)%pf->len) == 0) {
			if (((pe-p) < pf->len) 	/* Uncomplete field */
			   && (!p[strloc(p, '\n')])) break;
			errors += acheck(pf, p);
		    }
		}
	    }
	    if ((!fixed) && (*p == '\n')) {
		lineno++;
		if (ocount) icount(lsize, '\n');
		reclen = MAX (reclen, lsize);
		lsize = colno = colstart[0] = 0;
		continue;
	    }
	    if (ocount) {   /* Column Statistics */
		i = Xchar(*p);
		sums[i] += 1;
		if (fixed)      icount(total%lcount, i);
		else  if (lsize >= lcount) {
		    for (j=1; j < n; j++) if(p[j] == '\n') break;
		    /* Multiple of 128 columns */
		    memory (1+((lsize+j)|127));
		    icount(lsize, i);
		}
		else    icount(lsize, i);
	    }
	    else if (colsep == *p) {	/* Count number of columns */
		if (colno >= nfields) expand_fields(nfields+128);
		colwid(colno) += 1 ;
		colno++ ;
	    }
#if O_BINARY	/* For MS-DOS, don't take \r */
	    lsize += (*p != '\r');
#else
	    lsize++;
#endif
	}
	for (ps = buffer; p < pe; ) *(ps++) = *(p++);
	/* *ps = 0; printf(" <%s>+", buffer); */
	n = fread(ps, 1, (sizeof(buffer)-1)-(ps-buffer), infile);
	if (in_head) {
	    if (verbop) printf("....Realloc headbuf from %d to %d\n",
	       (int)lheadbuf, (int)(lheadbuf+n));
	    headbuf = realloc(headbuf, lheadbuf+n+1);
	    memcpy(headbuf+lheadbuf, ps, n);
	    lheadbuf += n;
	    headbuf[lheadbuf] = 0;
	}
	/* printf("%d\n", n); */
    }
    if (verbop) printf(llform[2], total);
    if ((k != _NL_) && ((ftyp == FC_AFILE)||(ftyp == FC_ACTRL)))
	printf("(doesn't end with \\n?)") ;
    if (ferror(infile)) perror(tschema);
    sums['\n'] = nls;

    if (fixed)	{		/* Number of records from reclen */
	nls = total/reclen;
	if (total%reclen) {	/* Size of last block */
	    nls++;
	    b = fits_dim + strlen(fits_dim);
	    if (fits_dim[0]) *(b++) = ' ';
	    sprintf(b, "lastBlock=%dbytes", (int)(total%reclen));
	}
	/*-- Pre-v6.73:
	reclen = total ;
	nls = 1 ;
	---*/
    }
    if (filinfo) {	/* Update FILINFO part */
    	filinfo->reclen = reclen;
    	filinfo->records = nls;
    }

    if (ftyp == FC_FITS) {
	printf(counts[FC_ABULK] == total ? " ascii " : " binary ");
    }
    else if ( (ftyp != FC_BINARY) && counts[FC_BINARY])
	printf (" +binary ");
    if (fits_dim[0]) printf("[%s] ", fits_dim);
    if ((ftyp == FC_BINARY) && (nls == 1) && (reclen > 999)) {
	nls = reclen;
	reclen = 1 ;
    }
    /* V6.71: Empty lines are OK for files not *.dat" */
    if (parseps && infile_name) {
	p = strstr(infile_name, ".dat");
	/* Empty lines are legal only for *.dat files.
	   There may howver be an added suffix in the case of
	   a compressed file.
	*/
	if (p && (!strchr(p, '/') )) ;	/* OK, keep the parseps value */
	else parseps = 0;
    }
    printf(": %d", (int)(nls-parseps));	/* V7.24: add headlines etc.. */
    if (heading) {
	printf("+%d", heading);
	if (hexp) printf("(%s)", hexp);
    }
    if (parseps) printf("+%db", parseps);
    printf(llform[3], reclen, total);
    if (colsep) {
	for (i=nfields-1; (i>=0) && (colwid(i) == 0); i--) ;
	printf(" %d cols:", i+1) ;
    }
    if ( (ftyp != FC_BINARY) && counts[FC_BINARY] && (ftyp != FC_FITS))
	printf (llform[4]/*" (%d non-ascii chars)"*/, counts[FC_BINARY]);
    if (hexp||parseps) {
	printf(" (");
        if (hexp) {
	    printf("%d headlines", heading);
	    if (parseps) printf(" + ");
	}
        if (parseps) printf("%d empty lines", parseps);
	printf(")");
    }

    if (fixed && (total%reclen))   printf(" (non-integer number of records ?)");

    /* Print the Ranges */
    if (fop&FOP_ra) {
    	printf("\n%s\n", rule('-'));
    	printf(" %7s %s  %-7s %-8s %s\n", " Bytes", "Format",
	    "Units", "Label", "[Range]");
    	printf("%s\n", rule('-'));
    	for (i=tform->nfmt, pf=tform->afmt; --i >= 0; pf++) {
   	    if (pf->b0 == pf->b1) printf(" %7d", pf->b0+1);
   	    else printf(" %3d-%3d", pf->b0+1, pf->b1+1);
   	    printf("  %-6s %-7s %-8s [%s]\n", fmt_ed(pf),
   	    	pf->unit, pf->label, fmt_edrange(pf));
    	}
    	printf("%s\n\n", rule('-'));
    }
    if (saved_more) { restore_more(saved_more); saved_more = 0; }
    if (cols2check) { free(cols2check); cols2check = 0; }

    if (!ocount)  {
  	free (ccounts); ccounts = (size_t *)0;
	return (ftyp);
    }

	/* List columns which are filled with the same char */

    size = MIN(reclen, MAXW);
    if (colsep) size = MIN(nfields, size) ;
    if (ou) {
	printf("\n\n#Constant columns:");
	for (j=0; j < size; j++) {
	    pcounts = &ccounts[j*128];
	    k = -1;
	    for (i = 0; (i<128) && (k<128); i++) {
		if (i == ' ')	continue;
		if (i == '\n')	continue;
		if (pcounts[i] == 0) continue;
		if (k >= 0)	/* 2 values not null */ k = 128;
		else k = i;
	    }
	    if (k < 128) {
		printf("\n#%4d = %s", j+1, pc(k));
		if ((k >= 0) && (pcounts['\n'] + pcounts[' ']))
			printf(" (or blank)");
	    }
	}
	printf("\n\n");
    }

	/* Write out the Column Statistics:
	*/

    if (!occ)  {
  	free (ccounts); ccounts = (size_t *)0;
	return (ftyp);
    }

    printf("\n\n#Column Statistics (decreasing frequency in each column):\n") ;
    printf("%s\n", rule('-'));
    for (j=ncols=0, pcounts=ccounts; j<size; j++, pcounts += 128) {
	m = 30 ;	/* Number of letters on 1 line */
	if (colsep) {
	    if (colwid(j) == 0) continue ;
	    m = 26 ;
	    ncols = j+1;
	    printf("%3d(%3d-%-3d):", ncols, colwid(j), colwnb(j)) ;
	}
    	else printf("%4d:", j+1);
    	for (i=0; i<128; i++) buffer[i] = i;
    	/* printf ("....Before sort:"); for (i=0; i<128; i++)
	   printf("%4d", pcounts[i]); putchar('\n'); */
    	qsort(buffer, 128, 1, (SORT_FCT)diffa);
    	/* printf ("....After  sort:"); for (i=0; i<128; i++)
	   printf("%4d", pcounts[buffer[i]]); putchar('\n'); */
    	for (i=0; buffer[i] && (i<m); i++) printf("%s", pc(buffer[i]));
	/* V7.0: save in pcounts[0] the number of different chars */
	pcounts[0] = i;
    	if (i < m) {
    	    k = i-1;
    	    while(i < m) printf("\t"), i += 4;
    	    printf("(%d", (int)pcounts[buffer[0]&0xff]);
    	    if (k != 0) printf(llform[5]/*"-%d"*/, pcounts[buffer[k]&0xff]);
    	    printf(")");
    	}
	else printf("...") ;
    	putchar('\n');
    }
    printf("%s\n", rule('-'));

    if (has_head) { 
	/* What are the column names and units ? */
	/* When header specified by a character (*hexp is a symbol),
	   the contents is ignored.
	   But if the header is specified by a number of lines:
	   -- first line = list of names
	   -- second line = units
	*/

	colnames=headbuf;
	if (/*hexp && */(lineno>hlines) && (ncols>0)/*(*hexp == 0)*/) {
	    for (p=headbuf, j=c=0, k=1; *p && (j<size); p++) {
		if(k) { colname(c) = p; c++; j++; k=0; }
		if (*p == colsep) { *p = 0; k=1; continue; }
		if (*p == '\n') {
		    if(c==ncols) break; 	/* End of headline */
		    colnames = p+1;
		    c=0; k=1;
		}
	    }
	    if (*p == '\n') *(p++) = 0;
	}
	if ((hlines > 1) && (lineno>hlines)) {
	    for (p=colnames, j=0, k=1; *p && (j<size); p++) {
		if(k) { colunit(j) = p; j++; k=0; }
		if (*p == '\n') break; 	/* End of headline */
		if (*p == colsep) { *p = 0; k=1; continue; }
	    }
	    if (*p == '\n') *(p++) = 0;
	}
    }

    if (colsep && (occ>1)) {	/* Propose the 'acut' transformations */
      char *radix="";		/* Indicates related column(s) */
	printf("\n#Proposed acut arguments to align the data:\n");
	if (infile_name) printf("fcat %s | ", infile_name);
	if (hexp) {
	    if (*hexp) ; 	/* printf(" grep -v '^%s' | ", hexp); */
	    else printf(" tail -n +%d | ", hlines+1);
	}
	printf("acut -d") ;
	if (colsep != '\t') printf("'%c'", colsep) ;
	if (hexp && *hexp) printf(" -C'%s'", hexp);	/* V7.21 */
	m = 0;			/* Used to indicate too large values */
	putchar(' ');
	nw = 0;			/* Counter to issue the newlines */
	for (j=0, pcounts=ccounts; j<size; j++, pcounts += 128) {
	    if (colwid(j) == 0) continue ;
	    k = 0;	/* Keep k=0: F ; k=1: A; k=2: I */
	    ab = 0;	/* Number of blanks to add for alignement */
	    for (i=1; (i<128) && (k == 0); i++) {
		if (pcounts[i] == 0) continue ;
		if (isspace(i)) continue ;
		k = ! acceptable_numeric[i];
	    }
	    if (colpec(j)&F_CHAR) k = 1;	/* Invalid numeric */
	    if ((nw&3) == 0) printf("\\\n  ");
	    if ((p = colname(j))) { if (p[1]=='_') p += 2; }
	    else { sprintf(p=buffer, "col.#%d", j+1); }
	    if (nw) printf(" -i'%c' ", strcmp(p, radix)==0 ? ' ' : '|'); 
	    else printf("       ");
	    nw++; radix = p;
	    if (j < 9) putchar(' ');
	    printf("-f%d", j+1); putchar('%') ;
	    if (k != 0) { putchar('-') ; k = 1; }
	    printf("%d", colwnb(j)) ;
	    if (k == 0) {
		if (coldec(j) == 0){k=2; ab=2;}	/* Integer number */
		else if ((colpec(j)&F_DBLE)==0)
		    printf(".%d", coldec(j)) ;
		else ab=2;
	    }
	    colpec(j) |= k ;	/* Keep column numeric/not */
	    putchar('j') ;
	    while (--ab >= 0) putchar(' ');
	    if (colwnb(j) < 10) putchar(' ');
	}

    }

    /* V7.0: Propose a format */
    if (occ>1) { int nc;
	printf("\n%s\n", rule('-'));
	printf("\n#Proposed format for the description of the data:\n") ;
	nw = 0;
	if (colsep) for (j=0, pcounts=ccounts; j<size; j++, pcounts += 128) {
	    if ((colwid(j)==0) && (!colname(j))) continue ;
	    k = colpec(j)&3 ;
	    if ((colpec(j)&F_DBLE) && (k==0)) ab = 'E';
	    else ab = fmt_letter[k];
	    printf("%cX %c%d", nw ? '1': '0', ab, colwnb(j)) ;
	    if (k == 0) {
		printf(".%d", coldec(j)) ;
	    }
	    else printf("  ") ;
	    nw++;				/* Running column number */
	    if (colwnb(j) < 10) putchar(' ');
	    /* Unit */
	    if ((p = colunit(j))) {
		while (*p == ' ') p++;
		if (!*p) p = (char *)0;
	    }
	    nc = printf(" %-6s", p ? p : " ---");
	    /* Column Name */
	    if ((p = colname(j))) {
		while (*p == ' ') p++;
		if (!*p) p = (char *)0;
	    }
	    if (p) nc += printf(" %-9s ", p);
	    else   nc += printf(" Col%-3d ", j+1);
	    if ((k&1)==0) {	/* Numeric --> Range */
		if (k == 2) {
		    if ((colmin(j) < -MAX_INT) || (colmax(j) > MAX_INT))
			nc += printf("**"), m++;
		    if (colmin(j) <= colmax(j))
			nc += printf( "[%.0f/%.0f]", colmin(j), colmax(j));
		}
		else if (colmin(j) <= colmax(j))
		    nc += printf("[%g/%g]", colmin(j), colmax(j));
	        if (colpec(j)&(F_NULL|F__nan|F__nul|F__inf|F__min)) { double f;
		    f = nls-parseps;
		    f = (100.*colnull(j))/f;
		    nc += printf("?%.1f%%", f);
		    if (colpec(j)&F__min) nc += printf("=---");
		    if (colpec(j)&F__nan) nc += printf("=NaN");
		    if (colpec(j)&F__inf) nc += printf("=Inf");
		    if (colpec(j)&F__nul) nc += printf("=null");
		}
	    }
	    /* if ((colpec(j)&F_NULL) && (k!=1)) putchar('?') ;
	    else */ putchar(' ');
	    while (nc<50) { putchar(' '); nc++; }
	    printf("Explain column #%d\n", j+1) ;
	}
	else {		/* Standard ASCII table, propose the format */
	  int trim = 0;	/* Number of blank columns which can be removed */
	    /* Usage: ab=number of blanks, i=number of fields */
	    i = j = k = m = 0;
	    pcounts = ccounts;
	    if ((pcounts[0]==1) && (pcounts[' '])) trim = 1;
	    while (j<size) {
		ab=0;		/* Number number of blank columns */
		while ((j<size) && (pcounts[0]==1) && (pcounts[' '])) {
		    j++; pcounts += 128;
		    ab++;
		}
		if (j>=size) continue;
		if (ab>0) trim += (ab-1); 	/* possible trim values */
		++i;		/* Field number */
		printf("%dX ", ab);
		/* Compute width of field (m) and its type (k)
		 * fixed = number of decimals
		 */
		m = ab = j; k = 2; fixed = -1;
		while (j<size) {
		    if((pcounts[0]==1) && (pcounts[' '])) break;
		    if (k&1); else for (c=' '; c<128; c++) {
			if (pcounts[c] == 0) continue;
			if (isspace(c)) continue;
			if (isdigit(c)) continue;
			if (issign(c)) continue;
			if (c == '.') { 	/* Decimal point */
			    k &= ~2;
			    if (fixed<0) fixed = j;
			    continue;
			}
			if (c == 'e') continue;
			if (c == 'E') continue;
			k |= 1;			/* Can't be a number */
		    }
		    j++; pcounts += 128;
		}
		putchar(fmt_letter[k]);
		m = j - m;
		if ((fixed<0)||(k&1))	 	/* No decimal point */
		    printf("%-5d", m);
		else {
		    fixed = j - fixed -1;
		    printf("%d.%-2d", m, fixed);
		    if (m<10) putchar(' ' );
		}
		printf(" ---  Col#%-3d  Explain cols %d-%d\n", i, ab+1, j);
	    }
	    m = 0;
	    if (trim) { 	/* Propose to remove useless blanks */
		/* Added V7.39 */
		pcounts = ccounts;
		i = j = 0;
		printf("\n# Blanks columns can be removed with:\n# trcol -b");
		if ((pcounts[0]==1) && (pcounts[' '])) { putchar('1'); i++; }
		while (j<size) {
		    ab=0;
		    while ((j<size) && (pcounts[0]==1) && (pcounts[' '])) {
			ab++; j++; pcounts += 128;
			if(ab>1) { if(i>0) putchar(','); printf("%d", j); i++; }
		    }
		    if (j>=size) continue;
		    while (j<size) {
			if((pcounts[0]==1) && (pcounts[' '])) break;
			j++; pcounts += 128;
		    }
		}
		putchar('\n');
	    }
	}
	if (m) printf("** indicates too Large values for Integer (x%d)\n", m);
    }

    free (ccounts); ccounts = (size_t *)0;
    return(ftyp);
}

#define pVal(text)      \
	if (! *++p) {                           	\
	    if (--argc <= 0) errors++, error(text);   	\
	    else    p = *++argv;            		\
	}
/*==================================================================
		Program
 *==================================================================*/
int main (int argc, char **argv)
{
  static char nouser[] = "www*\0nobody\0anonymous\0user\0"; /* V7.45 */
  FILINFO *pf, *pref; SECTION *ps;
  FILE *fp, *flist;
  char *tparm, **alist, *ulist[2];
  int nf = 0;
  int nrefs = 0 ;
  int init_done = 0;
  char *user = (char *)0;
  int i=0, n; char *p;
  char filename[256];

    setpgm("anafile");
    tparm = (char *)0 ; 	/* -t argument */
    /* From 'nobody' execution, diminish the number of warnings */
    if ((user = getenv("USER"))) { char *u;
	for (u=nouser; user && *u; u += strlen(u)+1) {
	    if (strmatch(u, user)) user = (char *)0;
	}
    }

    /* Check the Arguments */
    while (-- argc > 0) {
	p = *++argv;
	if (*p == '-') {
	  next_option:
	  ++p;
	  switch (*p) {
	    case 'w' :                   /* Set Max Width */
		pVal("Missing -w[idth] value");
		lcount = atoi(p);
		continue;
	    case 'F':
	    case 'd':
		colsep = p[1] ? p[1] : '\t';
		continue ;
	    case 'i':			/* Indirect fil.*/
		oi = 1 ;
		if (p[1]) {		/* e.g. -iinput */
		    p++ ;
		    break ;		/* goto FileArg */
		}
		continue ;
	    case 'q':                     /* Quiet mode   */
		oq   = 1;
		fop |= FOP_quiet;
		continue;
	    case 'v':                     /* Verbose      */
		if (strncmp(p, "ver", 3) == 0) {
		    printf("anafile (CDS) Version %s\n", VERSION);
		    exit(0);
		}
		++p;
		verbop = 1; fop |= FOP_v;
		if (isatty(1)) verbop |= 2 ;	/* Interactive 	*/
		continue;
	    case 'f':			/* Format-File	*/
		oa = 1 ;		/* Can't accept non-asci ! */
		while(*++p) switch(*p) {
		  case '.': fop |= FOP_dot; break;
		  case '1': fop |= FOP_1;   break;
		  case '2': fop |= FOP_2;   break;
		  case 's': fop |= FOP_std; break;
		  case 'w': fop |= FOP_w;   break;
		  case 'r': fop |= FOP_ra;  break;
		  case 'X':
		  case 'x': fop |= FOP_1X;  break;
		  case 'v': fop |= FOP_v;   verbop = 1; break;
		  case 'q': fop |= FOP_quiet; break;
		  default:
			fprintf(stderr, "\n****Bad format option: -%s\n", p);
			terminate((char *)0);
		}
		--p;
		pVal("Missing -f Format_File name");
		tinfo = cad_new(p, fop);	/* Format File */
		if (!tinfo) terminate("Errors in 'ReadMe' File, terminated");
		if (verbop) {			/* Edit the Sections */
		    printf("====Section Summary of File %s:\n", p);
    		    for (ps=tinfo->section[0]; ps; ps = ps->next)
    		        sec_prhead(ps, stdout);
		    show_files(tinfo);
		    show_tables(tinfo);
		}
		if (fop & (FOP_v|FOP_std)) 	/* Check Format File */
		    i = cad_check(tinfo);
		if (fop & FOP_std) {
		    if (i && user)
		      error("'ReadMe' file is not standard");
	    	}
		ofull = 1;
		/* Keeps the Directory Name */
		if (strchr(p, '/')) {
		    dReadMe = strdup(p);
		    *strrchr(dReadMe, '/') = 0;
		}
		continue;
	    case 'c':
		occ = ocount = 1;
		if (strchr(p, 'g')) occ++;
		if ((p[1] != 'c') && (p[1] != 'g')) goto bad_option;
		continue;
	    case 's': case 't':		/* Use table Structure	*/
	  	pVal("Missing -t(abular tructure) qualifier") ;
    		if (!tinfo) terminate("No ReadMe specified (-f option)");
		/* Specifying -t. or -t- stops the -t choice	*/
		tparm = isalnum(*p) ? p : (char *)0 ;
		continue ;
	    case '\0':			/* - alone = stdin	*/
		if (!init_done) init_done = 1, init();
		if (tparm) {
		    table_schema = tparm ;
		    printf("[%s]:", tparm);
		}
		infile_name = (char *)0;
		efile(stdin, ocount); putchar('\n');
		nf++ ;			/* Don't process tables	*/
		continue ;
	    case 'u':
		ocount = ou = 1;
		if (p[1]) goto next_option;
		continue;
	    case 'a':
		oa = 1;
		if(p[1] == 'l') ofull = 1;
		if (p[1]) goto next_option;
		continue;
	    case 'l':
		ofull = 1;
		if (p[1]) goto next_option;
		continue;
	    case 'h':
		if (strcmp(p, "help") == 0) {
		    help();
		    exit(0);
		}
		if (strcmp(p, "hash") == 0) {
		    headlines = 1;
		    headexp = "#";
		    continue;
		}
		if (strncmp(p, "head", 4) == 0) {
		    headlines = 1;
		    headexp = p + strloc(p, '=');
		    if (*headexp) 	/* Detail of HeadLines */
			headexp++;
		    if (isdigit(*headexp)) {
			headlines = atoi(headexp);
			while(isdigit(*headexp)) headexp++;
			headexp = (char *)0;
		    }
		    continue;
		}
	    default:
	    bad_option:
		fprintf(stderr, "\n****Bad option: -%s\n", p);
		errors++;
		terminate((char *)0);
	  }
	}

	/* ==== Here only for non-option (argument not starting by - */
        /*FileArgument:*/
#if DEBUG
	printf("Will look for file: %s", p); inbyte = getchar();
#endif
	if (!init_done) init_done = 1, init();
	/*if ((!isalnum(*p))&&(*p != '/')) { nf++; continue; } */
	if (is_dir(p)) {
	    if (oi) {
	    	fprintf(stderr, "**** Not a file: %s\n", p) ;
	    	terminate("Incompatible with -i option") ;
	    }
	    printf("%s/*: %s ", p, ftype(FC_DIRECTORY)) ;
	    if (ocount||ofull) dfile(p) ;
	    putchar('\n') ;
	    nf++ ;
	    continue ;
	}
	if (oi) {
	    flist = fopen(p, "r") ;
	    if (!flist)  perror(p), terminate("Stop");
	}
	else flist = (FILE *)0 ;
	while(1) {
	    if (flist) {	/* Get the List of Files */
	        if (!fgets(filename, sizeof(filename), flist)) break ;
	        for (p=filename+strlen(filename)-1;
	    	(p>=filename) && isspace(*p); p--) ;
	        *++p = 0 ;
	        p = filename ;
	    }
	    if (strchr(p, '*')) 	/* MULTIPLE FILE NAMES */
		alist = get_ls(p);
	    else {
		ulist[0] = p;
		ulist[1] = (char *)0;
		alist = ulist;
	    }
	    while ((p = *(alist++))) {
	        infile_name = p;
		pf = tinfo ? cad_findfile(tinfo, p) : (FILINFO *)0;
	        fp = fizopen (p);
	        if (!fp)  perror(p), terminate("Stop");
	        if (tparm)
	             table_schema = tparm, printf("[%s]", tparm) ;
	        else table_schema = p;
	        printf("%s:", p) ;
	        nf++;
	        fflush(stdout) ;
	        efile(fp, ocount);
		if (pf && pf->text) {	/* V7.24 */
		    putchar(' ');
		    fwrite(pf->text, 1, strloc(pf->text, '\n'), stdout);
		}
		putchar('\n');
	        fizclose (fp);
	        infile_name = (char *)0;
	    }
	    if (!oi) break ;
	}
	if (flist) fclose(flist), flist = (FILE *)0 ;
	continue;
    }
    /* if (errors)   { usage(); terminate((char *)0);} */
    if (!init_done) init_done = 1, init();

	/* If no file processed:
	   -> a standard 'ReadMe' file exists: use these files;
	   -> otherwise process on stdin
   	*/
    if (nf == 0) {
	if (tinfo && (!tparm)) {
	    nf = tinfo->nfinfo;
	    for(pf=tinfo->afinfo,i=nf; --i>=0; pf++) {
#if DEBUG
	        printf("....Dealing with: %s", pf->dfile); inbyte = getchar();
#endif
	        if (!pf->tabform) continue;
	        p  = pf->dfile;
	        if (strchr(p, '*')) {	/* MULTIPLE FILE NAMES */
		    alist = get_ls(p);
		    if (!alist) exit(1);
		}
	        else {
		    ulist[0] = p;
		    ulist[1] = (char *)0;
		    alist = ulist;
	        }
	        while ((p = *(alist++))) {
		    infile_name = p;
	            fp = fizopen (p);
	            if (!fp)  perror(p), terminate("Stop");
	            table_schema = p; nf++;
	            printf("%s:", p); fflush(stdout); efile(fp, ocount);
		    if (pf->text) {	/* V7.24 */
			putchar(' ');
			fwrite(pf->text, 1, strloc(pf->text, '\n'), stdout);
		    }
	            fizclose (fp); putchar('\n');
		    infile_name = (char *)0;
		}
	    }
	    /* Verify the References */
	    if ((pref = cad_findfile(tinfo, "refs.dat"))) {
		printf("----Verify the ") ; fflush(stdout) ;
		nrefs = load_refs(pref);    /* First Field must be Unique */
		if (nrefs > 0) printf
		    ("%d %s references from: %s\n", nrefs,
		      references.rint ? "(num.)" : "(char)", pref->dfile) ;
		for(pf=tinfo->afinfo,i=tinfo->nfinfo; --i>=0; pf++) {
		    if (! pf->tabform) continue ;
		    if (pf == pref) continue ;
		    printf("----%s: ", pf->dfile); fflush(stdout) ;
		    if (nrefs == 0) printf(" ++No Available References++\n") ;
		    else {
		        n = check_refs(pf) ;
			if (count_refcols > 0) printf(" [%d cols]",
			  count_refcols);
			/* if (count_nullrefs) printf(" (%dxNULL)",
			  count_nullrefs);
			*/
			if (n < 0) printf(" (--)\n") ;
		        else if (n == 0) printf(" (OK)\n") ;
		        else printf("\n****%s: %d inexistant references\n",
			    pf->dfile, n) ;
		    }
		}
	    }
	}
	if (nf == 0) {
	    if ((table_schema = tparm)) printf("[%s]", tparm) ;
	    printf(":"); 
	    if (fop&FOP_std)	/* V7.47: not if a standard ReadMe */
		printf(" no data file?");
	    else efile(stdin, ocount); 
	    putchar('\n');
	}
    }
    /* if (errors)   terminate((char *)0); */

    	/* Rewrite the "ReadMe" File */
    if (tinfo) {
#ifdef DEBUG
    	printf("\n====Edition of the '%s' file====", tinfo->ffile);
    	inbyte = getchar();
    	cad_print(tinfo, stdout, 0);
    	inbyte = getchar();
#endif
	if (fop&FOP_w) { /* Rewrite the file */
	  FILE *fout; TABINFO *pt;
	    /* V7.50: edition of headlines */
	    if (headlines || headexp) for (pt=tinfo->atinfo, i=tinfo->ntinfo; 
		    --i>=0; pt++) {
		if(headexp) pt->headchar = *headexp;
		else        pt->headlines = headlines;
	    }
	    p = malloc(2*strlen(tinfo->ffile) + 10);
#if O_BINARY	/* MS-DOS */
	    sprintf(p, "COPY %s %s~", tinfo->ffile, tinfo->ffile);
#else
	    sprintf(p, "mv %s %s~", tinfo->ffile, tinfo->ffile);
#endif
	    if (verbop&2) printf("....execute %s\n", p);
	    if (system(p) == -1) {
		fprintf(stderr, "****execute ") ;
		perror(p);
		fprintf(stderr, "    (reformatted file on standard output)\n");
		fout = stdout;
	    }
	    else fout = fopen(tinfo->ffile, "w");
	    free(p);
    	    if (!fout) {
		fprintf(stderr, "****");
		perror(tinfo->ffile);
		terminate("****Can't write format");
	    }
    	    else {
		cad_print(tinfo, fout, 0);
		if (fout != stdout) fclose(fout);
	    }
	}
#ifdef DEBUG
    	printf("====Will kill structures===="); inbyte = getchar();
#endif
	cad_kill(tinfo);
    }
    exit (errors != 0);
    exit(0);
}
